# Generated by Django 3.0.14 on 2022-04-12 22:47

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0314_punktacja_zrodla_kwarty"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0315_aktualna_jednostka_ostatnia_przypisana"
            )
        )
    ]



from django.urls import reverse


def test_RaportSlotow_get_pdf(admin_app, autor_jan_kowalski):
    url = reverse("raport_slotow:index")

    form_page = admin_app.get(url)
    form_page.forms[0]["obiekt"].force_value(autor_jan_kowalski.pk)

    raport_page = form_page.forms[0].submit().maybe_follow()

    pdf_page = raport_page.click("pobierz PDF")

    assert pdf_page.status_code == 200
    assert pdf_page.content[:8] == b"%PDF-1.7"



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0041_auto_20160722_2230'),
        ('bpp', '0046_ostatnia_jednostka_trigger'),
    ]

    operations = [
    ]



# Generated by Django 3.2.15 on 2022-08-15 15:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0005_auto_20220807_2329"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="zgloszenie_publikacji",
            name="dostep_dnia",
        ),
        migrations.RemoveField(
            model_name="zgloszenie_publikacji",
            name="public_dostep_dnia",
        ),
        migrations.RemoveField(
            model_name="zgloszenie_publikacji",
            name="public_www",
        ),
        migrations.RemoveField(
            model_name="zgloszenie_publikacji",
            name="www",
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="plik",
            field=models.FileField(
                blank=True,
                help_text="Jeżeli zgłaszana publikacja nie jest dostępna nigdzie w sieci internet,\n  "
                "      prosimy o dodanie załącznika",
                null=True,
                upload_to="",
                verbose_name="Plik załącznika",
            ),
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="strona_www",
            field=models.URLField(
                blank=True,
                help_text="Pole opcjonalne. Adres URL lokalizacji pełnego tekstu pracy (dostęp otwarty lub nie). "
                "Jeżeli praca posiada numer DOI, wpisz go w postaci adresu URL czyli https://dx.doi.org/"
                "[NUMER_DOI]. Jeżeli praca nie posiada numeru DOI bądź nie jest dostępna w sieci, pozostaw "
                "to pole puste. ",
                max_length=1024,
                null=True,
                verbose_name="Dostępna w sieci pod adresem",
            ),
        ),
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="status",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "nowe zgłoszenie"),
                    (1, "zaakceptowane - dodane do bazy BPP"),
                    (2, "wymaga zmian - odesłano do zgłaszającego"),
                    (3, "odrzucono w całości"),
                    (4, "spam"),
                ],
                default=0,
            ),
        ),
        migrations.DeleteModel(
            name="Zgloszenie_Publikacji_Plik",
        ),
    ]



import os
from decimal import Decimal

import openpyxl
from django.db.models import Sum, Value

from ewaluacja2021.const import LATA_2017_2018, LATA_2019_2021
from ewaluacja2021.reports import get_data_for_report, write_data_to_report
from ewaluacja2021.util import autor2fn, output_table_to_xlsx

from bpp.models import Autor


class WyjsciowyXLSX:
    def __init__(self, title, rekordy, dane, katalog_wyjsciowy):
        self.title = title
        self.rekordy = rekordy
        self.dane = dane
        self.katalog_wyjsciowy = katalog_wyjsciowy

        self.create_workbook()

    def create_workbook(self):
        self.wb = openpyxl.Workbook()

    def initialize_worksheet(self):
        self.ws = self.wb.active
        if self.title:
            self.ws.title = self.title[:31]

    def tabelka(self):
        write_data_to_report(self.ws, get_data_for_report(self.rekordy))

    def get_output_name(self):
        return f"{self.title}.xlsx"

    def zapisz(self):
        self.wb.save(os.path.join(self.katalog_wyjsciowy, self.get_output_name()))

    def metka(self):
        raise NotImplementedError()

    def zrob(self):
        self.initialize_worksheet()
        self.metka()
        self.ws.append([])
        self.tabelka()
        self.zapisz()


class CalosciowyXLSX(WyjsciowyXLSX):
    def metka(self):
        self.ws.append(
            [
                "Parametry raportu 3N",
                "raport całościowy",
            ]
        )
        self.ws.append(["Stan na dzień/moment", self.dane["ostatnia_zmiana"]])
        self.ws.append(["Dyscyplina", self.dane["dyscyplina"]])
        self.ws.append(["Liczba N", self.dane["liczba_n"]])
        self.ws.append(["Liczba 0.8N", self.dane["liczba_0_8_n"]])
        self.ws.append(["Liczba 2.2N", self.dane["liczba_2_2_n"]])
        self.ws.append(["Liczba 3*N", Decimal("3.0") * self.dane["liczba_n"]])
        self.ws.append(
            [
                "Suma slotów za lata 2017-2018",
                self.dane["sumy_slotow"][LATA_2017_2018],
            ]
        )
        self.ws.append(
            [
                "Suma slotów za lata 2019-2021",
                self.dane["sumy_slotow"][LATA_2019_2021],
            ]
        )

        sumy = self.rekordy.filter(do_ewaluacji=True).aggregate(
            suma_slot=Sum("slot"), suma_pkdaut=Sum("pkdaut")
        )
        self.ws.append(["Zebrana suma slotów za wszystkie prace", sumy["suma_slot"]])
        self.ws.append(["Zebrana suma PKDAut za wszystkie prace", sumy["suma_pkdaut"]])


class WypelnienieXLSX(CalosciowyXLSX):
    def get_data_for_report(self):
        id_autorow = self.rekordy.values_list("autor_id", flat=True).distinct()
        for autor in Autor.objects.filter(pk__in=id_autorow):

            maks_pkt_aut_calosc = self.dane["maks_pkt_aut_calosc"].get(str(autor.pk))
            maks_pkt_aut_monografie = self.dane["maks_pkt_aut_monografie"].get(
                str(autor.pk)
            )

            sumy = self.rekordy.filter(do_ewaluacji=True, autor_id=autor.pk).aggregate(
                suma_slot=Sum("slot"),
                suma_pkdaut=Sum("pkdaut"),
            )

            sumy_monografie = self.rekordy.filter(
                do_ewaluacji=True, monografia=Value("t"), autor_id=autor.pk
            ).aggregate(
                suma_slot=Sum("slot"),
            )

            sumy_wszystkie = self.rekordy.filter(autor_id=autor.pk).aggregate(
                suma_pkdaut=Sum("pkdaut"),
            )

            yield [
                str(autor.id),
                autor.nazwisko + " " + autor.imiona,
                maks_pkt_aut_calosc,
                sumy["suma_slot"] or 0,
                (sumy["suma_slot"] or 0) / maks_pkt_aut_calosc,
                maks_pkt_aut_monografie,
                (sumy_monografie["suma_slot"] or 0),
                (sumy_monografie["suma_slot"] or 0) / maks_pkt_aut_calosc,
                sumy["suma_pkdaut"],
                sumy_wszystkie["suma_pkdaut"],
                (sumy["suma_pkdaut"] or 0) / (sumy_wszystkie["suma_pkdaut"] or 1),
            ]

    def write_data_to_report(self, ws: openpyxl.worksheet.worksheet.Worksheet, data):
        output_table_to_xlsx(
            ws,
            "Przeszly",
            [
                # "ID elementu",
                "ID autora",
                "Nazwisko i imię",
                #
                "Maksymalna suma udziałów",
                "Sprawozdana suma udziałów",
                "Procent sprawozdanej sumy udziałów",
                #
                "Maksymalna suma udziałów - monografie",
                "Sprawozdana suma udziałów - monografie",
                "Procent sprawozdanej sumy udziałów - monografie",
                #
                "PKDaut prac sprawozdanych",
                "PKDaut wszystkich prac",
                "Procent PKDaut sprawozdanych",
            ],
            data,
            first_column_url="https://{site_name}/bpp/autor/",
            column_widths={
                "A": 10,
                "B": 14,
                "C": 14,
                "D": 14,
                "E": 14,
                "F": 14,
                "G": 14,
                "H": 14,
                "I": 14,
                "J": 14,
                "K": 14,
                "L": 14,
            },
            autor_column_url=1,
        )

    def tabelka(self):
        dane = self.get_data_for_report()
        self.write_data_to_report(self.ws, dane)


class AutorskiXLSX(WyjsciowyXLSX):
    def __init__(self, autor, title, rekordy, dane, katalog_wyjsciowy):
        super().__init__(
            title=title, rekordy=rekordy, dane=dane, katalog_wyjsciowy=katalog_wyjsciowy
        )
        self.autor = autor

    def metka(self):
        self.ws.append(
            [
                "Parametry raportu 3N",
                "wyciąg dla pojedynczego autora",
            ]
        )
        self.ws.append(["Stan na dzień/moment", self.dane["ostatnia_zmiana"]])
        self.ws.append(["Dyscyplina", self.dane["dyscyplina"]])
        self.ws.append(
            [
                "Maks. suma slotów za wszytkie prace",
                self.dane["maks_pkt_aut_calosc"].get(str(self.autor.pk)),
            ]
        )

        sumy = self.rekordy.filter(do_ewaluacji=True).aggregate(
            suma_slot=Sum("slot"), suma_pkdaut=Sum("pkdaut")
        )
        self.ws.append(["Zebrana suma slotów za wszystkie prace", sumy["suma_slot"]])
        self.ws.append(["Zebrana suma PKDAut za wszystkie prace", sumy["suma_pkdaut"]])

        self.ws.append(
            [
                "Maks. suma slotów za monografie",
                self.dane["maks_pkt_aut_monografie"].get(str(self.autor.pk)),
            ]
        )

        sumy = self.rekordy.filter(do_ewaluacji=True, monografia=Value("t")).aggregate(
            suma_slot=Sum("slot"), suma_pkdaut=Sum("pkdaut")
        )

        self.ws.append(["Zebrana suma slotów za monografie", sumy["suma_slot"]])
        self.ws.append(["Zebrana suma PKDAut za monografie", sumy["suma_pkdaut"]])

    def get_output_name(self):
        return autor2fn(self.autor) + ".xlsx"



# -*- encoding: utf-8 -*-


import importlib
import os
from django.core.management import BaseCommand
import sys
from bpp import models

class Command(BaseCommand):
    help = """Instaluje plik w danym obiekcie o danym PK
    Parametry:
    filename: nazwa pliku,
    model: nazwa modelu z bpp.models,
    pk: primary key tegoz modelu,
    field_name: pole, gdzie obraz zostanie wgrany
    """

    def handle(self, filename, model, pk, field_name, *args, **options):
        assert(os.path.exists(filename) and os.path.isfile(filename)), \
            "sciezka '%s' nie istnieje lub nie jest plikiem" % filename
        try:
            m = getattr(models, model)
        except AttributeError:
            print("Model '%s' nie zdefiniowany w bpp.models" % model)
            sys.exit(-1)

        try:
            obj = m.objects.get(pk=pk)
        except m.DoesNotExist:
            print("Obiekt '%s' o id %s nie istnieje." % (model, pk))
            sys.exit(-1)

        from django.core.files import File
        fld = getattr(obj, field_name)
        fld.save(os.path.basename(filename), File(open(filename, 'rb')))



class CodeAccessNotAllowed(Exception):
    """This exception occurs when a code tries to import a module from a
    path that is not explictly allowed.

    The way it works:
    * you should put ``DYNAMIC_COLUMNS_ALLOWED_IMPORT_PATHS`` setting in your settings.py,
      it should be a list of admin modules, that are allowed to be imported from module names
      stored inside database
    * if database wants to import a module that is not in this path, like instead importing
      ``yourapp.admin.SomeClass`` it wants to import ``malicious_module.code``, this exception
      will be raised.
    """



# Generated by Django 3.0.14 on 2021-05-19 00:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0267_auto_20210519_0237"),
    ]

    operations = [
        migrations.AlterField(
            model_name="uczelnia",
            name="pbn_aktualizuj_na_biezaco",
            field=models.BooleanField(
                default=False,
                help_text="Aktualizuj rekordy w PBN przy zapisie rekordu, gdy redaktor kliknie odpowiedni przycisk.\n        Wybranie tej opcji spowoduje, ze na podstronach modułu redagowania dla wydawnictw pojawią się\n        przyciski 'Zapisz i wyślij do PBN'. ",
                verbose_name="Włącz opcjonalną aktualizację przy edycji",
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2022-09-20 10:28

from django.db import migrations, models

import bpp.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CrossrefAPICache",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "doi",
                    bpp.fields.DOIField(
                        help_text="Digital Object Identifier (DOI)",
                        max_length=2048,
                        unique=True,
                    ),
                ),
                ("data", models.JSONField()),
                ("ostatnio_zmodyfikowany", models.DateTimeField(auto_now=True)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-02-21 22:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0117_auto_20180221_2327'),
    ]

    operations = [
        migrations.AlterField(
            model_name='typ_odpowiedzialnosci',
            name='typ_ogolny',
            field=models.SmallIntegerField(choices=[(0, 'autor'), (1, 'redaktor'), (2, 'inny'), (3, 'tłumacz'), (4, 'komentator'), (5, 'recenzent'), (6, 'opracował')], db_index=True, default=0, help_text='Pole to jest używane celem rozróżniania typu odpowiedzialności\n        na cele eksportu do PBN (autor i redaktor) oraz może być też wykorzystywane\n        np. w raportach autorów i jednostek. \n        ', verbose_name='Ogólny typ odpowiedzialności'),
        ),
    ]



from django.conf import settings
from django.db import models
from django.urls.base import reverse
from django.utils import timezone
from django.utils.html import escape
from django.utils.translation import ugettext_lazy as _

# Create your models here.
from model_utils.choices import Choices
from model_utils.fields import SplitField
from model_utils.models import StatusModel, TimeStampedModel

from bpp.models.struktura import Uczelnia

SPLIT_MARKER = getattr(settings, "SPLIT_MARKER", "WTF")


class Article(TimeStampedModel, StatusModel):
    STATUS = Choices(("draft", _("draft")), ("published", _("published")))

    title = models.TextField(verbose_name=_("Title"))
    article_body = SplitField(
        verbose_name=_("Article body"),
        help_text=_(
            'Use the split marker "%s" in case you want to display'
            "the shorter version of the article body"
        )
        % escape(SPLIT_MARKER),
    )
    published_on = models.DateTimeField(
        verbose_name=_("Published on"), default=timezone.now
    )
    slug = models.SlugField(unique=True)

    class Meta:
        verbose_name_plural = _("Articles")
        verbose_name = _("Article")
        ordering = ("-published_on", "title")

    def get_absolute_url(self):
        if self.status != self.STATUS.published:
            return reverse("admin:miniblog_article_change", args=(self.pk,))
        # TODO: co gdy będzie wiele uczelni w systemie?
        uczelnia = Uczelnia.objects.all().first()
        if self.article_body.has_more:
            return reverse("bpp:browse_artykul", args=(uczelnia.slug, self.slug))
        return reverse("bpp:browse_uczelnia", args=(uczelnia.slug,))

    def __str__(self):
        return f'Artykuł "{self.title}" - {self.STATUS[self.status]}'



import multiprocessing.pool as mpp


def istarmap(self, func, iterable, chunksize=1):
    """starmap-version of imap"""
    self._check_running()
    if chunksize < 1:
        raise ValueError("Chunksize must be 1+, not {0:n}".format(chunksize))

    task_batches = mpp.Pool._get_tasks(func, iterable, chunksize)
    result = mpp.IMapIterator(self)
    self._taskqueue.put(
        (
            self._guarded_task_generation(result._job, mpp.starmapstar, task_batches),
            result._set_length,
        )
    )
    return (item for chunk in result for item in chunk)


mpp.Pool.istarmap = istarmap



# Register your models here.
import json

from django.forms import widgets


class PrettyJSONWidget(widgets.Textarea):
    def format_value(self, value):
        try:
            value = json.dumps(json.loads(value), indent=2, sort_keys=True)
            # these lines will try to adjust size of TextArea to fit to content
            row_lengths = [len(r) for r in value.split("\n")]
            self.attrs["rows"] = min(max(len(row_lengths) + 2, 10), 60)
            self.attrs["cols"] = min(max(max(row_lengths) + 2, 40), 120)
            return value
        except Exception:
            # logger.warning("Error while formatting JSON: {}".format(e))
            return super(PrettyJSONWidget, self).format_value(value)



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models.patent import Patent, Patent_Autor
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle, \
    Wydawnictwo_Ciagle_Autor
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte, \
    Wydawnictwo_Zwarte_Autor


def znajdz_i_napraw_podwojnych(model, autor_model):
    rekord_pk = None
    poprzednia_kolejnosc = None

    for obj in autor_model.objects.all().order_by('rekord', 'kolejnosc'):
        # print autor_model, obj.rekord_id, obj.pk, obj.kolejnosc

        if obj.rekord_id != rekord_pk:
            rekord_pk = obj.rekord_id
            poprzednia_kolejnosc = None
            autorzy = []

            if obj.kolejnosc != 1:
                print("XX Dla rekordu %r %r autorzy zaczynaja sie od kolejnosci %i" % (
                    autor_model, obj.pk, obj.kolejnosc))

        if poprzednia_kolejnosc == obj.kolejnosc:
            print("XX Wykryto zdublowana kolejnosc %i dla wpisu %s %s" % (
                poprzednia_kolejnosc, autor_model, obj.pk))

        if poprzednia_kolejnosc is not None:
            delta = obj.kolejnosc - poprzednia_kolejnosc
            if delta != 1:
                print("XX roznica w kolejnosci wynosi %i dla rekordu %s %s" % (
                    delta, autor_model, obj.pk
                ))

        poprzednia_kolejnosc = obj.kolejnosc

        if obj.autor_id in autorzy:
            print("XX zdublowany autor %i dla rekordu %s %s (rekord_id %s)" % (
                obj.autor_id, autor_model, obj.pk, obj.rekord_id
            ))

            autorzy.append(obj.autor_id)



class Command(BaseCommand):
    help = 'Naprawia indeks autorow - szuka wpisów o tej samej kolejności'

    @transaction.atomic
    def handle(self, *args, **options):

        for model, autor_model in [
            (Wydawnictwo_Ciagle, Wydawnictwo_Ciagle_Autor),
            (Wydawnictwo_Zwarte, Wydawnictwo_Zwarte_Autor),
            (Patent, Patent_Autor)]:
            znajdz_i_napraw_podwojnych(model, autor_model)


# -*- encoding: utf-8 -*-
from hashlib import md5

import xlrd
from django.db import transaction

from .egeria_2012 import dopasuj_autora


def importuj_wiersz(dct, row, sheet, row_no, sheet_no):
    autor = dopasuj_autora(dct['IMIĘ'].value, dct['NAZWISKO'].value, dct['NAZWA JEDNOSTKI'].value, dct['Stanowisko'].value)
    if autor:
        autor.pesel_md5 = md5(dct['PESEL'].value).hexdigest()
        autor.pbn_id = dct['PBN ID'].value
        autor.save()
    # import pdb; pdb.set_trace()


def import_sheet(sheet, sheet_no, label_row_no=3):
    labels = [x.value for x in sheet.row(label_row_no-1)]
    for a in range(len(labels)):
        if labels[a] == '':
            labels[a] = str(a)

    for nrow in range(label_row_no, sheet.nrows):
        row = sheet.row(nrow)
        dct = dict(list(zip(labels, row)))
        importuj_wiersz(dct, row, sheet, nrow, sheet_no)


@transaction.atomic
def importuj_pbn_ids(plik_xls):
    # Tytuł/Stopień	NAZWISKO	IMIĘ	Wydział	Stanowisko	NAZWA JEDNOSTKI	PESEL	PBN ID

    book = xlrd.open_workbook(plik_xls)

    arkusze_ignorowane = ['zbiorczy', ]

    for cnt in range(book.nsheets):
        sheet = book.sheet_by_index(cnt)
        if sheet.name in arkusze_ignorowane:
            continue
        import_sheet(sheet, cnt)


if __name__ == "__main__":
    import sys

    import django; django.setup()
    importuj_pbn_ids(sys.argv[1])


# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-28 11:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0143_uczelnia_wydruk_logo_szerokosc'),
    ]

    # Ta migracja NIE może iść przez Django, musi być na poziomie SQL,
    # ponieważ faktyczna zmiana pola "Informacje" doprowadzi do zaktualizowania
    # daty w polu "Ostatnio zmieniony dla PBN" - jeżeli zrobimy to
    # przez Django. Jeżeli jednak zrobimy to przez SQL, to powinno
    # być w porządku...

    operations = [
        migrations.RunSQL(
            """
            UPDATE bpp_wydawnictwo_zwarte SET 
            informacje = TRIM(SUBSTR(informacje, 3))
            WHERE informacje ILIKE 'W:%';
            """,
            migrations.RunSQL.noop),
        migrations.RunSQL(
            """
            UPDATE bpp_patent SET 
            informacje = TRIM(SUBSTR(informacje, 3))
            WHERE informacje ILIKE 'W:%';
            """,
            migrations.RunSQL.noop),
        migrations.RunSQL(
            """
            UPDATE bpp_praca_doktorska SET 
            informacje = TRIM(SUBSTR(informacje, 3))
            WHERE informacje ILIKE 'W:%';
            """,
            migrations.RunSQL.noop),
        migrations.RunSQL(
            """
            UPDATE bpp_praca_habilitacyjna SET 
            informacje = TRIM(SUBSTR(informacje, 3))
            WHERE informacje ILIKE 'W:%';
            """,
            migrations.RunSQL.noop),



        migrations.RunSQL(
            """
            UPDATE bpp_wydawnictwo_zwarte SET 
            informacje = TRIM(SUBSTR(informacje, 4))
            WHERE informacje ILIKE 'W :%';
            """,
            migrations.RunSQL.noop),
        migrations.RunSQL(
            """
            UPDATE bpp_patent SET 
            informacje = TRIM(SUBSTR(informacje, 4))
            WHERE informacje ILIKE 'W :%';
            """,
            migrations.RunSQL.noop),
        migrations.RunSQL(
            """
            UPDATE bpp_praca_doktorska SET 
            informacje = TRIM(SUBSTR(informacje, 4))
            WHERE informacje ILIKE 'W :%';
            """,
            migrations.RunSQL.noop),
        migrations.RunSQL(
            """
            UPDATE bpp_praca_habilitacyjna SET 
            informacje = TRIM(SUBSTR(informacje, 4))
            WHERE informacje ILIKE 'W :%';
            """,
            migrations.RunSQL.noop),
    ]



from django.conf.urls import url

from .views import pobierz_z_crossref


class AdminCrossrefAPIMixin:
    crossref_templates = {}

    def get_urls(self):
        urls = super().get_urls()

        security_urls = [
            url(
                r"^pobierz-z-crossref/$",
                self.admin_site.admin_view(self.pobierz_z_crossref),
            )
        ]

        return security_urls + urls

    def pobierz_z_crossref(self, request):

        context = dict(
            self.admin_site.each_context(request),
            app_label="bpp",
            title="Pobierz z CrossRef",
        )
        return pobierz_z_crossref(request, context, self.crossref_templates)



import sys
import traceback
import uuid

from django.conf import settings
from django.contrib.messages import constants
from django.db import models, transaction
from django.urls import reverse
from django.utils import timezone

from long_running import const
from long_running.notification_mixins import NullNotificationMixin

TRACEBACK_LENGTH_LIMIT = 65535


class Operation(NullNotificationMixin, models.Model):
    """Długo działająca operacja"""

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)

    created_on = models.DateTimeField(auto_now_add=True)
    last_updated_on = models.DateTimeField(auto_now=True)

    started_on = models.DateTimeField(null=True, blank=True)
    finished_on = models.DateTimeField(null=True, blank=True)

    finished_successfully = models.BooleanField(default=False)
    traceback = models.TextField(null=True, blank=True)

    class Meta:
        ordering = ["-last_updated_on"]
        abstract = True

    def readable_exception(self):
        if self.traceback is None:
            return
        return [line for line in self.traceback.split("\n") if line][-1]

    def mark_started(self):
        # uruchamiać POZA transakcją

        self.started_on = timezone.now()
        self.save()
        self.send_notification(
            f"Rozpoczęto przetwarzanie, {self.started_on}", constants.INFO
        )

    def mark_finished_okay(self):
        # uruchamiać POZA transakcją
        self.finished_successfully = True
        self.finished_on = timezone.now()
        self.save()
        self.send_notification(
            f"Zakończono przetwarzanie - wszystko OK, {self.started_on}",
            constants.SUCCESS,
        )

    def mark_finished_with_error(self, exc_type, exc_value, exc_traceback):
        # uruchamiać POZA transakcją
        self.finished_successfully = False
        self.finished_on = timezone.now()
        self.traceback = "\n".join(
            traceback.format_exception(
                exc_type, exc_value, exc_traceback, limit=TRACEBACK_LENGTH_LIMIT
            )
        )
        self.save()

    @transaction.atomic
    def mark_reset(self):
        """Prepares the report for re-generation"""
        self.started_on = None
        self.finished_on = None
        self.finished_successfully = False
        self.traceback = None
        self.on_reset()
        self.save()

    def on_finished_successfully(self):
        pass

    def on_finished_with_error(self):
        self.send_notification(
            f"Zakończono z błędem, {self.finished_on}", constants.ERROR
        )

    def on_finished(self):
        self.send_processing_finished()

    def on_reset(self):
        pass

    def perform(self):
        raise NotImplementedError("Override this in a subclass.")

    def task_perform(self, raise_exceptions=True):
        """Runs a function in context of curret report, which means: it sets
        the variables according to success or failure of a given function.

        This function is meant to be called outside of a transaction,
        from a celery worker - only.
        """
        self.mark_started()
        try:
            with transaction.atomic():
                self.perform()
                self.mark_finished_okay()
            self.on_finished_successfully()
        except Exception:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.mark_finished_with_error(exc_type, exc_value, exc_traceback)
            self.on_finished_with_error()
            if raise_exceptions:
                raise exc_value.with_traceback(exc_traceback)
        finally:
            self.on_finished()

    redirect_prefix = None

    def get_redirect_prefix(self):
        """
        LongRunningOperationRouterView to widok, który decyduje, co zrobić z daną operacją.
        W tym celu korzysta z redirect_prefix czyli początku URLa, który domyślnie
        wygląda jak aplikacja+nazwa obiektu, czyli np:

            import_czegostam:importosob

        Do takiego redirect_prefix dodajemy suffix czyli np. -router, -details, -results
        i w ten sposob odsyłamy użytkownika na odpowiednią stronę - bądź to z monitorowaniem
        postępu danej operacji, bądź to z wynikami operacji.

        Domyślnie należy użytkownika odesłać na suffix -router, który to już
        potem decyduje, gdzie dalej odesłać przeglądarkę.
        """
        if self.redirect_prefix:
            return self.redirect_prefix
        return f"{self._meta.app_label}:{self._meta.model_name}"

    def get_url(self, suffix):
        return reverse(f"{self.get_redirect_prefix()}-{suffix}", args=(self.pk,))

    def get_absolute_url(self):
        return self.get_url("router")

    def get_state(self):
        if self.started_on is None:
            return const.PROCESSING_NOT_STARTED

        if self.started_on is not None and self.finished_on is None:
            return const.PROCESSING_STARTED

        if self.started_on is not None and self.finished_on is not None:
            if self.finished_successfully:
                return const.PROCESSING_FINISHED_SUCCESSFULLY
            return const.PROCESSING_FINISHED_WITH_ERROR

        raise NotImplementedError("This line should never execute")


class Report(Operation):
    def perform(self):
        self.create_report()

    def task_create_report(self, raise_exceptions=True):
        return self.task_perform(raise_exceptions=raise_exceptions)

    class Meta:
        abstract = True



# Generated by Django 2.1.7 on 2019-03-27 21:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0016_auto_20190327_0129'),
    ]

    operations = [
        migrations.AlterField(
            model_name='kolumna',
            name='rodzaj_pola',
            field=models.CharField(choices=[('tytuł', 'tytuł'), ('nazwisko', 'nazwisko'), ('imię', 'imię'), ('pesel', 'pesel'), ('orcid', 'orcid'), ('nazwa jednostki', 'nazwa jednostki'), ('wydział', 'wydział'), ('dyscyplina', 'dyscyplina'), ('kod dyscypliny', 'kod dyscypliny'), ('procent dyscypliny', 'procent dyscypliny'), ('subdyscyplina', 'subdyscyplina'), ('kod subdyscypliny', 'kod subdyscypliny'), ('procent subdyscypliny', 'procent subdyscypliny'), ('pomiń', 'pomiń')], max_length=50),
        ),
    ]



import django
import pytest
from django.urls import reverse

from bpp.models.dyscyplina_naukowa import Autor_Dyscyplina
from bpp.models.sloty.core import IPunktacjaCacher
from bpp.models.system import Charakter_Formalny
from bpp.models.wydawca import Wydawca
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from raport_slotow.views import RaportSlotowZerowy


@pytest.fixture
def fikstura_raportu_slotow(
    autor_jan_kowalski,
    autor_jan_nowak,
    jednostka,
    dyscyplina1,
    wydawnictwo_zwarte: Wydawnictwo_Zwarte,
    charaktery_formalne,
    typy_odpowiedzialnosci,
):
    # Nowak ma przypisanie na dany rok i ma prace
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak, rok=2020, dyscyplina_naukowa=dyscyplina1
    )

    wydawca = Wydawca.objects.create(nazwa="Wydawca")
    wydawca.poziom_wydawcy_set.create(rok=2020, poziom=1)

    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.charakter_formalny = Charakter_Formalny.objects.get(skrot="KSP")
    wydawnictwo_zwarte.rok = 2020
    wydawnictwo_zwarte.punkty_kbn = 100
    wydawnictwo_zwarte.dodaj_autora(
        autor=autor_jan_nowak,
        jednostka=jednostka,
        typ_odpowiedzialnosci_skrot="aut.",
        dyscyplina_naukowa=dyscyplina1,
    )

    wydawnictwo_zwarte.save()

    IPunktacjaCacher(wydawnictwo_zwarte).rebuildEntries()

    # Kowalski ma przypisanie i nie ma żadnych prac
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, rok=2020, dyscyplina_naukowa=dyscyplina1
    )

    return (autor_jan_kowalski, autor_jan_nowak)


def test_raport_slotow_zerowy_get_querylist_min_pk(fikstura_raportu_slotow):
    jk, jn = fikstura_raportu_slotow
    res = RaportSlotowZerowy(min_pk=200).get_queryset()
    res = res.values_list("autor_id", flat=True)
    assert res.count() == 2


def test_raport_slotow_zerowy_get_querylist(fikstura_raportu_slotow):
    jk, jn = fikstura_raportu_slotow

    res = RaportSlotowZerowy().get_queryset()

    res = res.values_list("autor_id", flat=True)

    assert res.count() == 1
    assert jk.id in res
    assert jn.id not in res


def test_raport_slotow_zerowy_rednering(
    fikstura_raportu_slotow, admin_client: django.test.Client
):
    res = admin_client.get(reverse("raport_slotow:raport-slotow-zerowy"))
    assert b"Kowalski" in res.content
    assert b"Nowak" not in res.content



# Generated by Django 3.2.15 on 2022-09-21 18:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0328_charakter_formalny_charakter_crossref"),
    ]

    operations = [
        migrations.AddField(
            model_name="jezyk",
            name="skrot_crossref",
            field=models.CharField(
                blank=True,
                choices=[("en", "en - angielski"), ("es", "es - hiszpański")],
                max_length=10,
                null=True,
                unique=True,
                verbose_name="Skrót nazwy języka wg API CrossRef",
            ),
        ),
        migrations.AlterField(
            model_name="charakter_formalny",
            name="charakter_crossref",
            field=models.PositiveSmallIntegerField(
                blank=True,
                choices=[(1, "journal-article"), (2, "proceedings-article")],
                default=None,
                null=True,
                unique=True,
                verbose_name="Charakter w CrossRef",
            ),
        ),
    ]



# Create your views here.
from django.http import HttpResponseBadRequest
from django.views.generic import RedirectView

from .client import OAuthMixin

from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin

from bpp.models import Uczelnia


class TokenRedirectPage(LoginRequiredMixin, RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        uczelnia = Uczelnia.objects.get_default()
        return OAuthMixin.get_auth_url(uczelnia.pbn_api_root, uczelnia.pbn_app_name)


class TokenLandingPage(LoginRequiredMixin, RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        ott = self.request.GET.get("ott")

        if not ott:
            raise HttpResponseBadRequest("Brak parametru OTT lub pusty")

        uczelnia = Uczelnia.objects.get_default()
        user_token = OAuthMixin.get_user_token(
            uczelnia.pbn_api_root, uczelnia.pbn_app_name, uczelnia.pbn_app_token, ott
        )

        user = self.request.user
        user.pbn_token = user_token
        user.save()

        messages.info(self.request, "Autoryzacja w PBN API przeprowadzona pomyślnie.")
        return "/"



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-16 08:51
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


def domyslne_bazy(apps, schema_editor):
    Zewnetrzna_Baza_Danych = apps.get_model("bpp", "Zewnetrzna_Baza_Danych")
    for nazwa, skrot in [
        ("Web of Science", "WOS"),
        ("Scopus", "SCOPUS")]:
        Zewnetrzna_Baza_Danych.objects.create(
            nazwa=nazwa, skrot=skrot
        )


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0125_auto_20180415_2223'),
    ]

    operations = [
        migrations.CreateModel(
            name='Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('info', models.CharField(blank=True, max_length=512, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Zewnetrzna_Baza_Danych',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa', models.CharField(max_length=512, unique=True)),
                ('skrot', models.CharField(max_length=128, unique=True)),
            ],
            options={
                'verbose_name': 'zewnętrzna baza danych',
                'verbose_name_plural': 'zenwętrzne bazy danych',
                'ordering': ['nazwa'],
            },
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle_zewnetrzna_baza_danych',
            name='baza',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Zewnetrzna_Baza_Danych'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle_zewnetrzna_baza_danych',
            name='rekord',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Wydawnictwo_Ciagle'),
        ),

        migrations.RunPython(
            domyslne_bazy,
            migrations.RunPython.noop
        )
    ]



from decimal import Decimal


class DecimalPathConverter:
    regex = "^(\d+(?:\.\d+)?)$"

    def to_python(self, value):
        # convert value to its corresponding python datatype
        return Decimal(value)

    def to_url(self, value):
        # convert the value to str data
        return str(value)



# chat/consumers.py
import json
from urllib.parse import parse_qs

from asgiref.sync import async_to_sync
from channels.generic.websocket import WebsocketConsumer

from notifications.core import get_channel_name_for_user
from notifications.models import Notification

from django.utils.functional import cached_property


class NotificationsConsumer(WebsocketConsumer):
    def _channels(self):
        # Kanał dla wszystkich - __all__
        yield "__all__"

        # Kanał wyłącznie dla konkretnego użytkownika
        if self.scope["user"].is_authenticated:

            yield get_channel_name_for_user(self.scope["user"])

            qstr = parse_qs(self.scope["query_string"])
            if b"extraChannels" in qstr:
                for elem in json.loads(qstr[b"extraChannels"][0]):
                    yield str(elem)

    @cached_property
    def channels(self):
        return list(self._channels())

    def subscribe(self):
        for channel in self.channels:
            async_to_sync(self.channel_layer.group_add)(channel, self.channel_name)

    def unsubscribe(self):
        for channel in self.channels:
            async_to_sync(self.channel_layer.group_discard)(channel, self.channel_name)

    def connect(self):
        self.subscribe()
        self.accept()

        Notification.objects.on_connect(self.channels)

    def disconnect(self, close_code):
        self.unsubscribe()

    def chat_message(self, event):
        self.send(text_data=json.dumps(event))

    def receive(self, text_data):
        text_data_json = json.loads(text_data)

        if text_data_json.get("type") == "ack_message":
            try:
                n = Notification.objects.get(id=text_data_json["id"])
            except Notification.DoesNotExist:
                return

            if n.channel_name in self.channels:
                n.acknowledge()



import pytest
from multiseek.logic import StringQueryObject

from bpp.models import BppMultiseekVisibility
from bpp.multiseek_registry.mixins import BppMultiseekVisibilityMixin


class TestQueryObject(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "foo"
    field_name = "bar"
    public = True


class TestQueryObjectDisabled(BppMultiseekVisibilityMixin, StringQueryObject):
    label = "foo"
    field_name = "bar"
    public = True

    def option_enabled(self):
        return False


@pytest.mark.django_db
def test_BppMultiseekVisibilityMixin_option_enabled_disabled():
    b = TestQueryObjectDisabled()
    assert not b.enabled(None)


@pytest.mark.django_db
def test_BppMultiseekVisibilityMixin_request_none():
    b = TestQueryObject()
    vis = b._get_or_create()

    assert b.enabled(None)

    vis.public = False
    vis.save()

    assert b.enabled(None) is False

    assert BppMultiseekVisibility.objects.count() == 1


def test_BppMultiseekVisibilityMixin_request_normal_user(rf, normal_django_user):
    b = TestQueryObject()

    vis = b._get_or_create()

    req = rf.get("/")
    req.user = normal_django_user

    vis.authenticated = False
    vis.save()
    assert not b.enabled(req)

    vis.authenticated = False
    vis.staff = True
    vis.save()
    assert not b.enabled(req)


def test_BppMultiseekVisibilityMixin_request_admin_user(rf, admin_user):
    b = TestQueryObject()

    vis = b._get_or_create()

    req = rf.get("/")
    req.user = admin_user

    vis.authenticated = False
    vis.staff = False
    vis.save()
    assert not b.enabled(req)

    vis.authenticated = True
    vis.staff = False
    vis.save()
    assert not b.enabled(req)

    vis.authenticated = False
    vis.staff = True
    vis.save()
    assert b.enabled(req)



from django.db import models
from django.db.models import DO_NOTHING

from bpp.fields import YearField
from bpp.models import BazaModeluOdpowiedzialnosciAutorow, TupleField


class RozbieznosciViewBase(models.Model):
    id = TupleField(models.IntegerField(), size=3, primary_key=True)
    rekord = models.ForeignKey("bpp.Rekord", DO_NOTHING, related_name="+")
    rok = YearField()
    autor = models.ForeignKey("bpp.Autor", DO_NOTHING, related_name="+")
    dyscyplina_rekordu = models.ForeignKey(
        "bpp.Dyscyplina_Naukowa", DO_NOTHING, related_name="+", null=True, blank=True
    )
    dyscyplina_autora = models.ForeignKey(
        "bpp.Dyscyplina_Naukowa", DO_NOTHING, related_name="+"
    )
    subdyscyplina_autora = models.ForeignKey(
        "bpp.Dyscyplina_Naukowa", DO_NOTHING, related_name="+", null=True, blank=True
    )

    class Meta:
        managed = False
        abstract = True


class BrakPrzypisaniaView(RozbieznosciViewBase):
    class Meta:
        managed = False


class RozbieznePrzypisaniaView(RozbieznosciViewBase):
    class Meta:
        managed = False


class RozbieznosciView(RozbieznosciViewBase):
    # Uwaga: w sytuacji, gdy praca będzie miała jednego i tego samego autora (np w roli redaoktora
    # oraz autora) to ten model i funkcja get_wydawnictwo_autor_obj zawiedzie.
    class Meta:
        managed = False
        verbose_name = "rozbieżność rekordu i dyscyplin"
        verbose_name_plural = "rozbieżności rekordów i dyscyplin"

    def get_wydawnictwo_autor_obj(self) -> BazaModeluOdpowiedzialnosciAutorow:
        # Uwaga: w sytuacji, gdy praca będzie miała jednego i tego samego autora (np w roli redaoktora
        # oraz autora) to ten model i funkcja get_wydawnictwo_autor_obj zawiedzie (zwraca wyłacznie pierwszy
        # rekord z powiazaniem autora + rekordu)
        return self.rekord.original.autorzy_set.filter(autor=self.autor).first()


class RozbieznosciZrodelView(models.Model):
    class Meta:
        managed = False
        verbose_name = "rozbieżność dyscyplin źródeł"
        verbose_name_plural = "rozbieżności dyscyplin źródeł"

    id = TupleField(models.IntegerField(), size=4, primary_key=True)
    zrodlo = models.ForeignKey("bpp.Zrodlo", on_delete=DO_NOTHING, related_name="+")
    wydawnictwo_ciagle = models.ForeignKey(
        "bpp.Wydawnictwo_Ciagle", on_delete=DO_NOTHING, related_name="+"
    )
    autor = models.ForeignKey("bpp.Autor", on_delete=DO_NOTHING, related_name="+")
    dyscyplina_naukowa = models.ForeignKey(
        "bpp.Dyscyplina_Naukowa", on_delete=DO_NOTHING, related_name="+"
    )



# Generated by Django 2.1.7 on 2019-08-06 18:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0171_auto_20190806_2015'),
    ]

    operations = [
        migrations.AlterField(
            model_name='poziom_wydawcy',
            name='rok',
            field=models.IntegerField(db_index=True),
        ),
        migrations.RunSQL("CREATE INDEX bpp_wydawnictwo_zwarte_wydawca_rok ON bpp_wydawnictwo_zwarte(wydawca_id, rok)"),
        migrations.RunSQL("CREATE INDEX bpp_praca_doktorska_wydawca_rok ON bpp_praca_doktorska(wydawca_id, rok)"),
        migrations.RunSQL("CREATE INDEX bpp_praca_habilitacyjna_wydawca_rok ON bpp_praca_habilitacyjna(wydawca_id, rok)"),
    ]



import datetime

import pytest
from django.conf import settings
from django.core.management import call_command
from django.test import TestCase
from model_bakery import baker

from bpp.models import (
    Charakter_Formalny,
    Jednostka,
    Jezyk,
    Patent,
    Patent_Autor,
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Status_Korekty,
    Typ_KBN,
    Typ_Odpowiedzialnosci,
    Tytul,
    Uczelnia,
    Wydawnictwo_Ciagle_Autor,
    Wydzial,
    Zrodlo,
    Zrodlo_Informacji,
)
from bpp.models.cache import Autorzy, AutorzyView, Rekord
from bpp.tests.tests_legacy.test_reports.util import autor, ciagle, zwarte
from bpp.tests.util import any_autor, any_ciagle

CHANGED = "foo-123-changed"


def ramka(msg):
    print("+" + ("-" * 78) + "+")
    print("+ " + msg)
    print("+" + ("-" * 78) + "+")


def clean_dict(ret):
    del ret["ostatnio_zmieniony"]
    del ret["tytul_oryginalny_sort"]
    return ret


class LoadFixturesMixin:
    def loadFixtures(self):
        for db_name in self._databases_names(include_mirrors=False):
            if hasattr(self, "fixtures"):
                call_command(
                    "loaddata",
                    *self.fixtures,
                    **{"verbosity": 0, "database": db_name, "skip_validation": True},
                )


class TestCacheMixin:
    # fixtures = [
    #     'typ_odpowiedzialnosci.json', 'tytul.json', 'zrodlo_informacji.json',
    #     'charakter_formalny.json', 'status_korekty.json', 'typ_kbn.json',
    #     'jezyk.json']

    def setUp(self):
        self.maxDiff = None

        aut = Typ_Odpowiedzialnosci.objects.get(skrot="aut.")
        self.typ_odpowiedzialnosci = aut

        self.uczelnia = baker.make(Uczelnia)
        self.wydzial = baker.make(Wydzial, uczelnia=self.uczelnia)
        self.j = baker.make(
            Jednostka, nazwa="Foo Bar", uczelnia=self.uczelnia, wydzial=self.wydzial
        )

        self.a = autor(self.j)
        self.a.nazwisko = "Kowalski"
        self.a.imiona = "Jan"
        self.a.tytul = Tytul.objects.get(skrot="dr")
        self.a.save()

        wspolne_dane = dict(
            adnotacje="adnotacje",
            informacja_z=Zrodlo_Informacji.objects.all()[0],
            status_korekty=Status_Korekty.objects.all()[0],
            rok=2000,
            www="http://127.0.0.1/",
            recenzowana=True,
            impact_factor=5,
            punkty_kbn=5,
            index_copernicus=5,
            punktacja_wewnetrzna=5,
            weryfikacja_punktacji=True,
            typ_kbn=Typ_KBN.objects.all()[0],
            jezyk=Jezyk.objects.all()[0],
            informacje="informacje",
            szczegoly="szczegoly",
            uwagi="uwagi",
            slowa_kluczowe="slowa kluczowe",
        )

        zwarte_dane = dict(
            miejsce_i_rok="Lublin 2012",
            wydawnictwo="Pholium",
            redakcja="Redkacja",
            isbn="isbn",
            e_isbn="e_isbn",
            tytul="tytul",
        )

        self.z = zwarte(
            self.a,
            self.j,
            aut,
            tytul_oryginalny="zwarte",
            liczba_znakow_wydawniczych=40000,
            charakter_formalny=Charakter_Formalny.objects.all()[0],
            **dict(list(zwarte_dane.items()) + list(wspolne_dane.items())),
        )

        self.zr = baker.make(Zrodlo, nazwa="Zrodlo")

        self.c = ciagle(
            self.a,
            self.j,
            tytul_oryginalny="ciągłe",
            zrodlo=self.zr,
            tytul="tytul",
            issn="issn",
            e_issn="e_issn",
            charakter_formalny=Charakter_Formalny.objects.all()[0],
            **wspolne_dane,
        )
        self.assertEqual(Wydawnictwo_Ciagle_Autor.objects.all().count(), 1)

        wca = Wydawnictwo_Ciagle_Autor.objects.all()[0]
        wca.typ_odpowiedzialnosci = self.typ_odpowiedzialnosci
        wca.save()

        settings.BPP_CACHE_ENABLED = True

        # Doktorat i habilitacja

        doktorat_kw = dict(list(zwarte_dane.items()) + list(wspolne_dane.items()))

        self.d = baker.make(
            Praca_Doktorska,
            tytul_oryginalny="doktorat",
            autor=self.a,
            jednostka=self.j,
            **doktorat_kw,
        )

        self.h = baker.make(
            Praca_Habilitacyjna,
            tytul_oryginalny="habilitacja",
            autor=self.a,
            jednostka=self.j,
            **doktorat_kw,
        )

        # Patent

        Charakter_Formalny.objects.get(skrot="PAT")

        for elem in ["typ_kbn", "jezyk"]:
            del wspolne_dane[elem]

        self.p = baker.make(
            Patent,
            tytul_oryginalny="patent",
            numer_zgloszenia="100",
            data_decyzji=datetime.date(2012, 1, 1),
            **wspolne_dane,
        )

        Patent_Autor.objects.create(
            autor=self.a,
            jednostka=self.j,
            rekord=self.p,
            typ_odpowiedzialnosci=aut,
            zapisany_jako="Kowalski",
        )

        self.wszystkie_modele = [self.d, self.h, self.p, self.c, self.z]


class TestCacheRebuildBug(TestCase):
    def test_liczba_znakow_bug(self):
        Rekord.objects.full_refresh()
        self.assertEqual(Rekord.objects.all().count(), 0)

        any_ciagle(tytul="foo", liczba_znakow_wydawniczych=31337)
        Rekord.objects.full_refresh()

        self.assertEqual(Rekord.objects.all().count(), 1)
        self.assertEqual(Rekord.objects.all()[0].tytul, "foo")
        self.assertEqual(Rekord.objects.all()[0].liczba_znakow_wydawniczych, 31337)


class TestCacheSimple(TestCacheMixin, TestCase):
    def setUp(self):
        Typ_Odpowiedzialnosci.objects.get_or_create(skrot="aut.", nazwa="autor")
        Charakter_Formalny.objects.get_or_create(skrot="PAT")
        for skrot, nazwa in [("ang.", "angielski"), ("fr.", "francuski")]:
            Jezyk.objects.get_or_create(skrot=skrot, nazwa=nazwa)
        for klass in [Typ_KBN, Zrodlo_Informacji, Status_Korekty]:
            baker.make(klass)

        super().setUp()

    def test_get_original_object(self):
        Rekord.objects.full_refresh()
        for model in self.wszystkie_modele:
            c = Rekord.objects.get_original(model)
            self.assertEqual(c.original, model)

    def test_cache_triggers(self):
        T1 = "OMG ROXX"
        T2 = "LOL"

        for model in self.wszystkie_modele:
            model.tytul_oryginalny = T1
            model.save()
            self.assertEqual(Rekord.objects.get_original(model).tytul_oryginalny, T1)

            model.tytul_oryginalny = T2
            model.save()
            self.assertEqual(Rekord.objects.get_original(model).tytul_oryginalny, T2)

    def assertInstanceEquals(self, instance, values_dict):
        for key, value in list(values_dict.items()):
            instance_value = getattr(instance, key)
            self.assertEqual(
                instance_value,
                value,
                msg=f"key={key}, {value}!={instance_value}",
            )

    def test_tytul_sorted_version(self):
        for elem in [self.d, self.h, self.c, self.z]:  # self.p]:
            elem.tytul_oryginalny = "The 'APPROACH'"
            elem.jezyk = Jezyk.objects.get(skrot="ang.")
            elem.save()

            self.assertEqual(
                Rekord.objects.get_original(elem).tytul_oryginalny_sort, "approach"
            )

            elem.tytul_oryginalny = "le 'test'"
            elem.jezyk = Jezyk.objects.get(skrot="fr.")
            elem.save()

            # elem = elem.__class__.objects.get(pk=elem.pk) #reload
            # self.assertEquals(elem.tytul_oryginalny_sort, "test")

            self.assertEqual(
                Rekord.objects.get_original(elem).tytul_oryginalny_sort, "test"
            )


class TestCacheZapisani(LoadFixturesMixin, TestCase):
    fixtures = ["typ_odpowiedzialnosci.json"]

    def test_zapisani_wielu(self):
        aut = any_autor("Kowalski", "Jan")
        aut2 = any_autor("Nowak", "Jan")

        baker.make(Uczelnia)
        jed = baker.make(Jednostka)
        wyd = any_ciagle(tytul_oryginalny="Wydawnictwo ciagle")

        for kolejnosc, autorx in enumerate([aut, aut2]):
            Wydawnictwo_Ciagle_Autor.objects.create(
                autor=autorx,
                jednostka=jed,
                rekord=wyd,
                typ_odpowiedzialnosci_id=1,
                zapisany_jako="FOO BAR",
                kolejnosc=kolejnosc,
            )

        Rekord.objects.full_refresh()
        c = Rekord.objects.get_original(wyd)

        # Upewnij się, że w przypadku pracy z wieloma autorami do cache
        # zapisywane jest nie nazwisko z pól 'zapisany_jako' w bazie danych,
        # a oryginalne
        self.assertEqual(
            c.opis_bibliograficzny_autorzy_cache, ["Kowalski Jan", "Nowak Jan"]
        )

        # Upewnij się, że pole 'opis_bibliograficzny_zapisani_autorzy_cache'
        # zapisywane jest prawidłowo
        self.assertEqual(
            c.opis_bibliograficzny_zapisani_autorzy_cache, "FOO BAR, FOO BAR"
        )

    def test_zapisani_jeden(self):
        aut = any_autor("Kowalski", "Jan")
        baker.make(Uczelnia)
        dok = baker.make(Praca_Doktorska, tytul_oryginalny="Doktorat", autor=aut)

        Rekord.objects.full_refresh()
        c = Rekord.objects.get_original(dok)

        # Upewnij się, że w przypadku pracy z jednym autorem do cache
        # zapisywana jest prawidłowa wartość
        self.assertEqual(c.opis_bibliograficzny_autorzy_cache, ["Kowalski Jan"])

        self.assertEqual(c.opis_bibliograficzny_zapisani_autorzy_cache, "Kowalski Jan")


@pytest.mark.django_db
def test_MinimalCachingProblem_tworzenie(
    statusy_korekt, jezyki, typy_odpowiedzialnosci
):
    def foo():
        j = baker.make(Jednostka)
        a = any_autor()

        assert Autorzy.objects.all().count() == 0

        c = any_ciagle(impact_factor=5, punktacja_wewnetrzna=0)
        assert Rekord.objects.all().count() == 1

        c.dodaj_autora(a, j)

        assert AutorzyView.objects.all().count() == 1
        assert Autorzy.objects.all().count() == 1

    foo()


@pytest.mark.django_db
def test_MinimalCachingProblem_usuwanie(statusy_korekt, jezyki, typy_odpowiedzialnosci):
    def foo():
        j = baker.make(Jednostka)
        a = any_autor()

        assert Autorzy.objects.all().count() == 0

        c = any_ciagle(impact_factor=5, punktacja_wewnetrzna=0)
        assert Rekord.objects.all().count() == 1

        c.dodaj_autora(a, j)

        c.delete()

        assert AutorzyView.objects.all().count() == 0
        assert Autorzy.objects.all().count() == 0

    foo()



from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import Autor_Jednostka
from bpp.util import pbar


class Command(BaseCommand):
    help = "Przebudowuje przypisania autora do jednostek w całej bazie; wskazane uruchamianie raz na dobę"

    @transaction.atomic
    def handle(self, *args, **options):
        for elem in pbar(Autor_Jednostka.objects.all()):
            elem.save()



from django.test import TestCase
from model_bakery import baker

from bpp.models import POLA_PUNKTACJI, Wydawnictwo_Ciagle
from bpp.models.abstract import ILOSC_ZNAKOW_NA_ARKUSZ


class AbstractModelsTestCase(TestCase):
    def test_pola(self):
        self.assertIn("impact_factor", POLA_PUNKTACJI)
        self.assertNotIn("weryfikacja_punktacji", POLA_PUNKTACJI)

    def test_model_punktowany(self):
        mp = baker.make(Wydawnictwo_Ciagle, impact_factor=0)
        self.assertEqual(mp.ma_punktacje(), False)

        mp.impact_factor = 0
        self.assertEqual(mp.ma_punktacje(), False)

        mp.impact_factor = 1
        self.assertEqual(mp.ma_punktacje(), True)

    def test_ModelZeZnakamiWydawniczymi(self):
        x = baker.make(Wydawnictwo_Ciagle, liczba_znakow_wydawniczych=None)

        # x.liczba_znakow_wydawniczych = None
        self.assertTrue(not x.ma_wymiar_wydawniczy())

        x.liczba_znakow_wydawniczych = 5
        self.assertTrue(x.ma_wymiar_wydawniczy())

        x.liczba_znakow_wydawniczych = None
        self.assertFalse(x.ma_wymiar_wydawniczy())

        x.liczba_znakow_wydawniczych = 5
        self.assertTrue(x.ma_wymiar_wydawniczy())

        x.liczba_znakow_wydawniczych = ILOSC_ZNAKOW_NA_ARKUSZ
        self.assertEqual(x.wymiar_wydawniczy_w_arkuszach(), "1.00")

        x.liczba_znakow_wydawniczych = ILOSC_ZNAKOW_NA_ARKUSZ * 2.5
        self.assertEqual(x.wymiar_wydawniczy_w_arkuszach(), "2.50")



from celery import shared_task

from rozbieznosci_dyscyplin.admin import real_ustaw_dyscypline


@shared_task
def task_ustaw_dyscypline(pole, elements):
    from rozbieznosci_dyscyplin.admin import ResultNotifier

    notifier = ResultNotifier()
    real_ustaw_dyscypline(pole, elements, notifier)
    return notifier.retbuf



# Generated by Django 2.1.7 on 2019-06-26 22:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0159_auto_20190327_0028'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent_autor',
            name='dyscyplina_naukowa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle_autor',
            name='dyscyplina_naukowa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte_autor',
            name='dyscyplina_naukowa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AlterField(
            model_name='charakter_formalny',
            name='charakter_pbn',
            field=models.ForeignKey(blank=True, default=None, help_text='Wartość wybrana w tym polu zostanie użyta jako zawartość tagu &lt;is>\n                                      w plikach eksportu do PBN', null=True, on_delete=django.db.models.deletion.CASCADE, to='bpp.Charakter_PBN', verbose_name='Charakter PBN'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-14 17:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0124_auto_20180414_1917'),
        ('import_dyscyplin', '0006_auto_20180414_1841'),
    ]

    operations = [
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='dyscyplina_naukowa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='subdyscyplina_naukowa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AlterField(
            model_name='import_dyscyplin_row',
            name='stan',
            field=models.CharField(choices=[('nowy', 'nowy'), ('błędny', 'błędny'), ('zintegrowany', 'zintegrowany')], db_index=True, default='nowy', max_length=50),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-08 12:44
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import django_fsm
import import_dyscyplin.models
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Import_Dyscyplin',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('plik', models.FileField(upload_to='')),
                ('rok', models.IntegerField(default=import_dyscyplin.models.obecny_rok)),
                ('stan', django_fsm.FSMField(choices=[('nowy', 'nowy'), ('przeanalizowany', 'przeanalizowany'), ('błędny', 'błędny'), ('dyscypliny zmatchowane', 'dyscypliny zmatchowane'), ('wydziały zmatchowane', 'wydziały zmatchowane'), ('jednostki zmatchowane', 'jednostki zmatchowane'), ('autorzy zmatchowani', 'autorzy zmatchowani'), ('plik zintegrowany', 'plik zintegrowany')], default='nowy', max_length=50, protected=True)),
                ('bledny', models.BooleanField(default=False)),
                ('info', models.TextField(blank=True, null=True)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0019_charakter_formalny_charakter_pbn'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='charakter_pbn',
            options={'ordering': ['identyfikator'], 'verbose_name': 'Charakter PBN', 'verbose_name_plural': 'Charaktery PBN'},
        ),
        migrations.AlterField(
            model_name='charakter_formalny',
            name='charakter_pbn',
            field=models.ForeignKey(on_delete=models.CASCADE, default=None, blank=True, to='bpp.Charakter_PBN', null=True, verbose_name=b'Charakter PBN'),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.0.14 on 2021-11-01 22:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tee", "0002_auto_20211031_2122"),
    ]

    operations = [
        migrations.AddField(
            model_name="log",
            name="traceback",
            field=models.TextField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-08 19:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('celeryui', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='report',
            name='file',
            field=models.FileField(blank=True, null=True, upload_to='report'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-17 11:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0130_clarivate_analytics'),
    ]

    operations = [
        migrations.AddField(
            model_name='praca_doktorska',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Liczba cytowań'),
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Liczba cytowań'),
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("notifications", "0002_notification_acknowledged"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="notification",
            options={"ordering": ("created_on",)},
        ),
        migrations.AlterField(
            model_name="notification",
            name="acknowledged",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AlterField(
            model_name="notification",
            name="values",
            field=models.JSONField(),
        ),
    ]



# Generated by Django 2.1.10 on 2019-10-27 10:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0186_auto_20191021_2008'),
    ]

    operations = [
        migrations.AddField(
            model_name='praca_doktorska',
            name='jezyk_alt',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Jezyk', verbose_name='Język alternatywny'),
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='jezyk_alt',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Jezyk', verbose_name='Język alternatywny'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='jezyk_alt',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Jezyk', verbose_name='Język alternatywny'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='jezyk_alt',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='bpp.Jezyk', verbose_name='Język alternatywny'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-09-10 20:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0179_auto_20190910_1416'),
        ('bpp', '0179_auto_20190910_2147'),
    ]

    operations = [
    ]



# Generated by Django 3.0.14 on 2021-05-30 00:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0270_auto_20210530_0114"),
    ]

    operations = [
        migrations.AddField(
            model_name="patent_autor",
            name="profil_orcid",
            field=models.BooleanField(
                default=False,
                help_text="Zaznacz, jeżeli praca znajdje się na profilu ORCID autora",
                verbose_name="Praca w profilu ORCID autora",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle_autor",
            name="profil_orcid",
            field=models.BooleanField(
                default=False,
                help_text="Zaznacz, jeżeli praca znajdje się na profilu ORCID autora",
                verbose_name="Praca w profilu ORCID autora",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte_autor",
            name="profil_orcid",
            field=models.BooleanField(
                default=False,
                help_text="Zaznacz, jeżeli praca znajdje się na profilu ORCID autora",
                verbose_name="Praca w profilu ORCID autora",
            ),
        ),
        migrations.AlterField(
            model_name="autor",
            name="orcid_w_pbn",
            field=models.NullBooleanField(
                help_text="Jeżeli ORCID jest w bazie PBN, to pole powinno być zaznaczone. Zaznaczenie następuje\n        automatycznie, przez procedury integrujące bazę danych z PBNem w nocy. Można też zaznaczyć ręcznie.\n        Pole wykorzystywane jest gdy autor nie ma odpowiednika w PBN (pole 'PBN UID' rekordu autora jest puste,\n        zaś eksport danych powoduje komunikat zwrotny z PBN o nieistniejącym w ich bazie ORCID). W takich sytuacjach\n        należy w polu wybrać \"Nie\". ",
                verbose_name="ORCID jest w bazie PBN?",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-15 09:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0009_auto_20180415_0847'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='import_dyscyplin',
            options={'ordering': ('-modified',)},
        ),
        migrations.AlterField(
            model_name='import_dyscyplin_row',
            name='dyscyplina',
            field=models.CharField(db_index=True, max_length=200),
        ),
        migrations.AlterField(
            model_name='import_dyscyplin_row',
            name='kod_dyscypliny',
            field=models.CharField(db_index=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='import_dyscyplin_row',
            name='kod_subdyscypliny',
            field=models.CharField(blank=True, db_index=True, max_length=20, null=True),
        ),
        migrations.AlterField(
            model_name='import_dyscyplin_row',
            name='subdyscyplina',
            field=models.CharField(blank=True, db_index=True, max_length=200, null=True),
        ),
    ]



import json

from crossref.restful import Works
from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import Rekord


class Command(BaseCommand):
    help = "Pobiera dane z Crossref API dla ostatnich 100 rekordów z DOI"

    @transaction.atomic
    def handle(self, *args, **options):
        works = Works()
        for rekord in (
            Rekord.objects.exclude(doi=None)
            .exclude(doi="")
            .order_by("-ostatnio_zmieniony")[:100]
        ):
            data = works.doi(rekord.doi)
            if data:
                print(f"{rekord.tytul_oryginalny}")
                print(f"{rekord.pk}")
                print("")
                print(json.dumps(data, indent=4))
                print("-" * 80)



import pytest
from django.urls import reverse

from dynamic_columns.models import ModelAdmin, ModelAdminColumn

from django.contrib.admin import site
from django.contrib.contenttypes.models import ContentType


def test_autor_admin_hide_column(admin_app, autor_jan_kowalski):
    POPRZEDNIE_NAZWISKA = "159ygquadfja0eth0qjaoidjgo"

    autor_jan_kowalski.poprzednie_nazwiska = POPRZEDNIE_NAZWISKA
    autor_jan_kowalski.save()

    # Open the admin, so DynamicColumnMixin.enabled will be called
    res = admin_app.get(reverse("admin:bpp_autor_changelist"))
    assert "Kowalski" in res

    # Get the instance of AutorAdmin
    autor_admin = site._registry.get(autor_jan_kowalski.__class__)

    ma = ModelAdmin.objects.enable(autor_admin)
    c = ma.modeladmincolumn_set.get(col_name="poprzednie_nazwiska")
    c.enabled = True
    c.save()

    # Open the 'default' admin instance
    res = admin_app.get(reverse("admin:bpp_autor_changelist"))
    assert POPRZEDNIE_NAZWISKA in res
    assert "Kowalski" in res

    ma = ModelAdmin.objects.db_repr(autor_admin)
    c = ma.modeladmincolumn_set.get(col_name="poprzednie_nazwiska")
    c.enabled = False
    c.save()

    res = admin_app.get(reverse("admin:bpp_autor_changelist"))
    assert POPRZEDNIE_NAZWISKA not in res
    assert "Kowalski" in res


@pytest.mark.django_db
def test_ModelAdminColumn___str__():

    b = ModelAdminColumn(col_name="bar")
    assert str(b) == 'Column "bar"'

    a = ModelAdmin.objects.create(
        class_name="foo", model_ref=ContentType.objects.all().first()
    )
    b.parent = a

    assert str(b) == 'Column "bar" of model "foo"'



from django.utils.deprecation import MiddlewareMixin
from messages_extends.models import Message

class NotificationsMiddleware(MiddlewareMixin):
    def process_request(self, request):
        """After entering a web page, we may want to mark some messages as read
        basing on the URL they contain.
        """

        # request.user may be a SimpleLazyObject instance
        try:
            user_id = request.user.pk
        except AttributeError:
            return

        if user_id is None:
            return

        url = request.get_full_path()
        Message.objects.filter(user_id=user_id, read=False, message__icontains=url).update(read=True)



"""
Małe klasy pomocnicze dla całego systemu
"""
import warnings

from django.db import models
from django.db.models import CASCADE
from django.db.models.signals import post_migrate
from django.dispatch import receiver
from django.urls import reverse
from model_utils import Choices
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel

from bpp import const
from bpp.models.abstract import ModelZNazwa, NazwaISkrot

NAZWY_PRIMO = [
    "",
    "Artykuł",
    "Książka",
    "Zasób tekstowy",
    "Rozprawa naukowa",
    "Recenzja",
    "Artykuł prasowy",
    "Rozdział",
    "Czasopismo",
    "Dane badawcze",
    "Materiał konferencyjny",
    "Obraz",
    "Baza",
    "Zestaw danych statystycznych",
    "Multimedia",
    "Inny",
]

NAZWY_PRIMO = list(zip(NAZWY_PRIMO, NAZWY_PRIMO))

RODZAJE_DOKUMENTOW_PBN = [
    ("article", "Artykuł"),
    ("book", "Książka"),
    ("chapter", "Rozdział"),
]


class Charakter_PBN(models.Model):
    wlasciwy_dla = models.CharField(
        "Właściwy dla...", max_length=20, choices=RODZAJE_DOKUMENTOW_PBN
    )
    identyfikator = models.CharField(max_length=100)
    opis = models.CharField(max_length=500)
    help_text = models.TextField(blank=True)

    class Meta:
        ordering = ["identyfikator"]
        verbose_name = "Charakter PBN"
        verbose_name_plural = "Charaktery PBN"

    def __str__(self):
        return self.opis


CHARAKTER_SLOTY = Choices(
    (const.CHARAKTER_SLOTY_KSIAZKA, "ksiazka", "Książka"),
    (const.CHARAKTER_SLOTY_ROZDZIAL, "rozdzial", "Rozdział"),
    (const.CHARAKTER_SLOTY_REFERAT, "referat", "Referat"),
)

RODZAJ_PBN_CHOICES = [
    (None, "nie eksportuj do PBN"),
    (const.RODZAJ_PBN_ARTYKUL, "artykuł"),
    (const.RODZAJ_PBN_KSIAZKA, "książka"),
    (const.RODZAJ_PBN_ROZDZIAL, "rozdział"),
    (const.RODZAJ_PBN_POSTEPOWANIE, "postępowanie"),
]

CHARAKTER_OGOLNY_CHOICES = Choices(
    (const.CHARAKTER_OGOLNY_ARTYKUL, "artykul", "Artykuł"),
    (const.CHARAKTER_OGOLNY_KSIAZKA, "ksiazka", "Książka"),
    (const.CHARAKTER_OGOLNY_ROZDZIAL, "rozdzial", "Rozdział"),
    (const.CHARAKTER_OGOLNY_INNE, "inne", "Inne"),
)


class Charakter_Formalny(NazwaISkrot, MPTTModel):
    class CHARAKTER_CROSSREF(models.IntegerChoices):
        JOURNAL_ARTICLE = 1, "journal-article"
        PROCEEDINGS_ARTICLE = 2, "proceedings-article"

    denorm_always_skip = (
        "tree_id",
        "lft",
        "rght",
        "level",
    )
    parent = TreeForeignKey(
        "self",
        verbose_name="Charakter nadrzędny",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name="children",
    )

    charakter_ogolny = models.CharField(
        max_length=3,
        help_text="""Charakter ogólny uzywany jest m.in. do generowania opisów bibliograficznych. Stanowi on
        ogólne określenie rekordu, czy jest to książka, rozdział czy coś innego. """,
        default=const.CHARAKTER_OGOLNY_INNE,
        choices=CHARAKTER_OGOLNY_CHOICES,
    )

    publikacja = models.BooleanField(
        help_text="""Jest charakterem dla publikacji""", default=False
    )
    streszczenie = models.BooleanField(
        help_text="""Jest charakterem dla streszczeń""", default=False
    )

    nazwa_w_primo = models.CharField(
        "Nazwa w Primo",
        max_length=100,
        help_text="""
    Nazwa charakteru formalnego w wyszukiwarce Primo, eksponowana przez OAI-PMH. W przypadku,
    gdy to pole jest puste, prace o danym charakterze formalnym nie będą udostępniane przez
    protokół OAI-PMH.
    """,
        blank=True,
        default="",
        choices=NAZWY_PRIMO,
        db_index=True,
    )

    charakter_pbn = models.ForeignKey(
        Charakter_PBN,
        verbose_name="Charakter PBN",
        blank=True,
        null=True,
        default=None,
        help_text="""Wartość wybrana w tym polu zostanie użyta jako zawartość tagu &lt;is>
                                      w plikach eksportu do PBN""",
        on_delete=CASCADE,
    )

    rodzaj_pbn = models.PositiveSmallIntegerField(
        verbose_name="Rodzaj dla PBN",
        choices=RODZAJ_PBN_CHOICES,
        null=True,
        blank=True,
        help_text="""Pole określające, czy wydawnictwa posiadające dany charakter formalny zostaną włączone
        do eksportu PBN jako artykuły, rozdziały czy książki. """,
        default=None,
    )

    charakter_sloty = models.PositiveSmallIntegerField(
        "Charakter dla slotów",
        null=True,
        blank=True,
        default=None,
        choices=CHARAKTER_SLOTY,
        help_text="""Jak potraktować ten charakter przy kalkulacji slotów dla wydawnictwa zwartego?""",
    )

    charakter_crossref = models.PositiveSmallIntegerField(
        "Charakter w CrossRef",
        null=True,
        blank=True,
        default=None,
        choices=CHARAKTER_CROSSREF.choices,
        unique=True,
    )

    wliczaj_do_rankingu = models.BooleanField(default=True)

    class Meta:
        ordering = ["nazwa"]
        app_label = "bpp"
        verbose_name = "charakter formalny"
        verbose_name_plural = "charaktery formalne"

    class MPTTMeta:
        order_insertion_by = ["nazwa"]

    #
    # Kompatybilne API dla .artykul_pbn, .rozdzial_pbn, .ksiazka_pbn
    #

    def get_rodzaj(self, typ):
        warnings.warn("W przyszlosci uzyj pola 'rodzaj_pbn'", DeprecationWarning)
        v = getattr(const, "RODZAJ_PBN_%s" % typ.upper())
        if self.rodzaj_pbn == v:
            return True
        return False

    def set_rodzaj(self, typ, value):
        warnings.warn("W przyszlosci uzyj pola 'rodzaj_pbn'", DeprecationWarning)
        v = getattr(const, "RODZAJ_PBN_%s" % typ.upper())
        if value is True:
            self.rodzaj_pbn = v
        else:
            raise NotImplementedError(
                "Nie wiem jak sie zachowac, gdy atrybut self.%s_pbn jest ustawiany na False"
                % typ
            )

    def get_artykul_pbn(self):
        return self.get_rodzaj("artykul")

    def get_ksiazka_pbn(self):
        return self.get_rodzaj("ksiazka")

    def get_rozdzial_pbn(self):
        return self.get_rodzaj("rozdzial")

    def set_artykul_pbn(self, v):
        return self.set_rodzaj("artykul", v)

    def set_ksiazka_pbn(self, v):
        return self.set_rodzaj("ksiazka", v)

    def set_rozdzial_pbn(self, v):
        return self.set_rodzaj("rozdzial", v)

    artykul_pbn = property(get_artykul_pbn, set_artykul_pbn)
    ksiazka_pbn = property(get_ksiazka_pbn, set_ksiazka_pbn)
    rozdzial_pbn = property(get_rozdzial_pbn, set_rozdzial_pbn)

    def get_absolute_url(self):
        return reverse("admin:bpp_charakter_formalny_change", args=(self.pk,))


@receiver(post_migrate)
def rebuild_handler(sender, **kwargs):
    # Ponieważ przechodzimy z modelu bez-MPTT na model z-MPTT, wypełniamy
    # mu defaultowe wartości dla poziomu, parents, itp. Stąd też, po migracji
    # potrzebne jest przebudowanie obiektów, aby relacje rodzic-dziecko
    # realizowane byłyprawidłowo
    if sender.name == "bpp":
        Charakter_Formalny.objects.rebuild()


class Status_Korekty(ModelZNazwa):
    class Meta:
        verbose_name = "status korekty"
        verbose_name_plural = "statusy korekty"
        app_label = "bpp"


class Zrodlo_Informacji(ModelZNazwa):
    class Meta:
        verbose_name = "źródło informacji o bibliografii"
        verbose_name_plural = "źródła informacji o bibliografii"
        app_label = "bpp"


class Typ_Odpowiedzialnosci(NazwaISkrot):
    typ_ogolny = models.SmallIntegerField(
        "Ogólny typ odpowiedzialności",
        choices=[
            (const.TO_AUTOR, "autor"),
            (const.TO_REDAKTOR, "redaktor"),
            (const.TO_INNY, "inny"),
            (const.TO_TLUMACZ, "tłumacz"),
            (const.TO_KOMENTATOR, "komentator"),
            (const.TO_RECENZENT, "recenzent"),
            (const.TO_OPRACOWAL, "opracował"),
            (const.TO_REDAKTOR_TLUMACZENIA, "redaktor tłumaczenia"),
        ],
        default=const.TO_AUTOR,
        help_text="""Pole to jest używane celem rozróżniania typu odpowiedzialności
        na cele eksportu do PBN (autor i redaktor) oraz może być też wykorzystywane
        np. w raportach autorów i jednostek.
        """,
        db_index=True,
    )

    class Meta:
        verbose_name = "typ odpowiedzialności"
        verbose_name_plural = "typy odpowiedzialności"
        ordering = ["nazwa"]
        app_label = "bpp"

    def __str__(self):
        return self.nazwa


class Jezyk(NazwaISkrot):
    class SKROT_CROSSREF(models.TextChoices):
        en = "en", "en - angielski"
        es = "es", "es - hiszpański"

    skrot_crossref = models.CharField(
        max_length=10,
        verbose_name="Skrót nazwy języka wg API CrossRef",
        choices=SKROT_CROSSREF.choices,
        blank=True,
        null=True,
        unique=True,
    )

    pbn_uid = models.ForeignKey(
        "pbn_api.Language", null=True, blank=True, on_delete=models.SET_NULL
    )

    class Meta:
        verbose_name = "język"
        verbose_name_plural = "języki"
        ordering = ["nazwa"]
        app_label = "bpp"

    def get_skrot_dla_pbn(self):
        if self.skrot_dla_pbn:
            return self.skrot_dla_pbn

        return self.skrot


class Typ_KBN(NazwaISkrot):
    artykul_pbn = models.BooleanField(
        "Artykuł w PBN",
        help_text="""Wydawnictwa ciągłe posiadające
    ten typ KBN zostaną włączone do eksportu PBN jako artykuły""",
        default=False,
    )

    charakter_pbn = models.ForeignKey(
        Charakter_PBN,
        verbose_name="Charakter PBN",
        blank=True,
        null=True,
        default=None,
        help_text="""Wartość wybrana w tym polu zostanie użyta jako
        fallback, tzn. jeżeli dla charakteru formalnego danego rekordu nie
        określono odpowiedniego charakteru PBN, to zostanie użyta wartość
        tego pola, o ile wybrana. """,
        on_delete=CASCADE,
    )

    wliczaj_do_rankingu = models.BooleanField(default=True)

    class Meta:
        verbose_name = "typ KBN"
        verbose_name_plural = "typy KBN"
        ordering = ["nazwa"]
        app_label = "bpp"


class Rodzaj_Prawa_Patentowego(ModelZNazwa):
    class Meta:
        verbose_name = "rodzaj prawa patentowego"
        verbose_name_plural = "rodzaje praw patentowych"
        ordering = [
            "nazwa",
        ]
        app_label = "bpp"


class Zewnetrzna_Baza_Danych(NazwaISkrot):
    class Meta:
        verbose_name = "zewnętrzna baza danych"
        verbose_name_plural = "zenwętrzne bazy danych"
        ordering = [
            "nazwa",
        ]
        app_label = "bpp"



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-20 00:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0098_wybitnosc'),
    ]

    operations = [
        migrations.AddField(
            model_name='nagroda',
            name='nazwa',
            field=models.CharField(default='', max_length=200),
            preserve_default=False,
        ),
    ]



# -*- encoding: utf-8 -*-

import logging

import xlrd
from django.core.management import BaseCommand
from django.core.exceptions import ValidationError

from bpp.models import Wydawca

logger = logging.getLogger("django")


class Command(BaseCommand):
    help = "Importuje aliasy wydawców z pliku XLS (alias -> nazwa wydawcy)"

    def add_arguments(self, parser):
        parser.add_argument("plik")

    def handle(
        self, plik, verbosity, *args, **options,
    ):
        logger.setLevel(logging.WARNING)
        if verbosity > 1:
            logger.setLevel(logging.INFO)

        book = xlrd.open_workbook(plik)
        sheet = book.sheet_by_index(0)

        for row_idx in range(1, sheet.nrows):
            cells = sheet.row_slice(rowx=row_idx, start_colx=0, end_colx=2)
            alias, wydawca = [cell.value for cell in cells]

            try:
                w = Wydawca.objects.get(nazwa=wydawca.strip())
            except Wydawca.DoesNotExist:
                logger.info(f"{wydawca} zdefiniowany w pliku XLS nie istnieje")
                continue

            a = Wydawca.objects.get_or_create(nazwa=alias.strip())[0]

            if a.alias_dla != w:
                a.alias_dla = w
                try:
                    a.save()
                except ValidationError:
                    logger.warn(
                        f"** alias {alias} rozpoznany jako {a} to alias do samego siebie {w}, idę dalej"
                    )
                    continue
                logger.info(f"{w.nazwa} dopisuje alias {a.nazwa}")
            else:
                logger.info(f"{w.nazwa} już miał {a.nazwa}")



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 11:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0086_calkowita_liczba_redaktorow'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_index_copernicus',
            field=models.BooleanField(default=True, verbose_name='Pokazuj Index Copernicus na stronie rekordu'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_punktacje_wewnetrzna',
            field=models.BooleanField(default=True, verbose_name='Pokazuj punktację wewnętrzną na stronie rekordu'),
        ),
    ]



from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static
from django.urls import path
from django.views.decorators.cache import cache_page
from django.views.decorators.csrf import csrf_exempt
from django.views.i18n import JavaScriptCatalog
from loginas.views import user_login
from password_policies.views import (
    PasswordChangeDoneView,
    PasswordChangeFormView,
    PasswordResetCompleteView,
    PasswordResetConfirmView,
    PasswordResetDoneView,
    PasswordResetFormView,
)

from django.contrib import admin
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import LoginView, LogoutView

from bpp.forms import MyAuthenticationForm
from bpp.views import favicon, root
from bpp.views.admin import (
    PatentTozView,
    WydawnictwoCiagleTozView,
    WydawnictwoZwarteTozView,
)
from bpp.views.global_nav import global_nav_redir
from bpp.views.mymultiseek import (
    MyMultiseekResults,
    bpp_remove_by_hand,
    bpp_remove_from_removed_by_hand,
)

from django_bpp.forms import BppPasswordChangeForm

admin.autodiscover()


urlpatterns = (
    [
        url(r"^favicon\.ico$", cache_page(60 * 60)(favicon)),
        url(
            r"^admin/bpp/wydawnictwo_ciagle/toz/(?P<pk>[\d]+)/$",
            login_required(WydawnictwoCiagleTozView.as_view()),
            name="admin_bpp_wydawnictwo_ciagle_toz",
        ),
        url(
            r"^admin/bpp/wydawnictwo_zwarte/toz/(?P<pk>[\d]+)/$",
            login_required(WydawnictwoZwarteTozView.as_view()),
            name="admin_bpp_wydawnictwo_ciagle_toz",
        ),
        url(
            r"^admin/bpp/patent/toz/(?P<pk>[\d]+)/$",
            login_required(PatentTozView.as_view()),
            name="admin_bpp_wydawnictwo_ciagle_toz",
        ),
        # url(r'^admin/', include(admin.site.urls)),
        path("admin/", admin.site.urls),
        url(
            r"^integrator2/",
            include(("integrator2.urls", "integrator2"), namespace="integrator2"),
        ),
        path(
            "ewaluacja2021/",
            include(
                "ewaluacja2021.urls",
            ),
        ),
        url(
            r"^api/v1/",
            include(("api_v1.urls", "api_v1"), namespace="api_v1"),
        ),
        path(
            "zglos_publikacje/",
            include(
                ("zglos_publikacje.urls", "zglos_publikacje"),
            ),
        ),
        path(
            "pbn_api/",
            include("pbn_api.urls"),
        ),
        url(
            r"^import_pracownikow/",
            include(
                ("import_pracownikow.urls", "import_pracownikow"),
                namespace="import_pracownikow",
            ),
        ),
        url(
            r"^import_list_if/",
            include(
                ("import_list_if.urls", "import_list_if"),
                namespace="import_list_if",
            ),
        ),
        url(
            r"^import_dyscyplin/",
            include(
                ("import_dyscyplin.urls", "import_dyscyplin"),
                namespace="import_dyscyplin",
            ),
        ),
        url(
            r"^nowe_raporty/",
            include(("nowe_raporty.urls", "nowe_raporty"), namespace="nowe_raporty"),
        ),
        path("raport_slotow/", include("raport_slotow.urls")),
        url(r"^bpp/", include(("bpp.urls", "bpp"), namespace="bpp")),
        path("rozbieznosci_dyscyplin/", include("rozbieznosci_dyscyplin.urls")),
        path("rozbieznosci_if/", include("rozbieznosci_if.urls")),
        url(
            r"^multiseek/results/$",
            csrf_exempt(
                MyMultiseekResults.as_view(
                    registry=settings.MULTISEEK_REGISTRY,
                    template_name="multiseek/results.html",
                )
            ),
            name="multiseek:results",
        ),
        url(
            r"^multiseek/",
            include(("multiseek.urls", "multiseek"), namespace="multiseek"),
        ),
        url(
            r"^multiseek/live-results/$",
            csrf_exempt(
                MyMultiseekResults.as_view(
                    registry=settings.MULTISEEK_REGISTRY,
                    template_name="multiseek/live-results.html",
                )
            ),
            name="live-results",
        ),
        url(
            r"^multiseek/remove-from-results/(?P<pk>\w+)$",
            bpp_remove_by_hand,
            name="remove_from_results",
        ),
        url(
            r"^multiseek/remove-from-removed-results/(?P<pk>\w+)$",
            bpp_remove_from_removed_by_hand,
            name="remove_from_removed_results",
        ),
        url(r"^admin_tools/", include("admin_tools.urls")),
        url(r"^grappelli/", include("grappelli.urls")),
        url(r"^$", root, name="root"),
        url(
            r"^accounts/login/$",
            LoginView.as_view(authentication_form=MyAuthenticationForm),
            name="login_form",
        ),
        url(
            r"^password_change_done/$",
            PasswordChangeDoneView.as_view(),
            name="password_change_done",
        ),
        url(
            r"^password_change/$",
            PasswordChangeFormView.as_view(form_class=BppPasswordChangeForm),
            name="password_change",
        ),
        url(
            r"^password_reset/$", PasswordResetFormView.as_view(), name="password_reset"
        ),
        url(
            r"^password_reset_confirm/"
            r"([0-9A-Za-z_\-]+)/([0-9A-Za-z]{1,13})/([0-9A-Za-z-=_]{1,64})/$",
            PasswordResetConfirmView.as_view(),
            name="password_reset_confirm",
        ),
        url(
            r"^password_reset_done/$",
            PasswordResetDoneView.as_view(),
            name="password_reset_done",
        ),
        url(
            r"^password_reset_complete/$",
            PasswordResetCompleteView.as_view(),
            name="password_reset_complete",
        ),
        url(r"^logout/$", LogoutView.as_view(), name="logout"),
        url(
            r"^messages/",
            include(
                ("messages_extends.urls", "messages_extends"),
                namespace="messages_extends",
            ),
        ),
        url(
            r"^.*/jsi18n/$",
            JavaScriptCatalog.as_view(
                packages=[
                    "multiseek",
                ]
            ),
        ),
        url(r"session_security/", include("session_security.urls")),
        url(r"^login/user/(?P<user_id>.+)/$", user_login, name="loginas-user-login"),
        url(r"^robots\.txt", include("robots.urls")),
        # url(r'^sitemap\.xml$', cache_page(7*24*3600)(sitemaps_views.index), {
        #     'sitemaps': django_bpp_sitemaps,
        #     'sitemap_url_name': 'sitemaps'
        # }, name='sitemap'),
        # url(r'^sitemap-(?P<section>.+)\.xml$',
        #     cache_page(7*24*3600)(sitemaps_views.sitemap), {'sitemaps': django_bpp_sitemaps},
        #     name='sitemaps'),
        # url(r'^sitemap\.xml', include('static_sitemaps.urls')),
        path("", include("static_sitemaps.urls")),
        url(r"", include("webmaster_verification.urls")),
        url(
            r"^global-nav-redir/(?P<param>.+)/$",
            global_nav_redir,
            name="global-nav-redir",
        ),
    ]
    + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
    + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
)


if settings.DEBUG and settings.DEBUG_TOOLBAR:
    import debug_toolbar

    urlpatterns += [
        url(r"^__debug__/", include(debug_toolbar.urls)),
    ]

handler404 = "bpp.views.handler404"
handler500 = "bpp.views.handler500"
handler403 = "bpp.views.handler403"



import json

import openpyxl
from django.core.management import call_command


def test_genetyczny(genetyczny_3n):
    """
    Duży test integracyjny, sprawdza generowanie raportu genetycznego
    3N na bazie jednej pracy.
    """

    dyscyplina1 = genetyczny_3n["dyscyplina"]
    call_command("raport_3n_genetyczny", dyscyplina=dyscyplina1.nazwa)

    raport = json.loads(open("genetyczny_memetyka_stosowana.json").read())
    assert len(raport["optimum"]) == 10


def test_genetyczny_zapis_xlsx(genetyczny_3n):
    dyscyplina1 = genetyczny_3n["dyscyplina"]
    call_command("raport_3n_genetyczny", dyscyplina=dyscyplina1.nazwa)
    call_command("raport_3n_to_xlsx", "genetyczny_memetyka_stosowana.json")

    res = openpyxl.load_workbook(
        "genetyczny_memetyka_stosowana_output/AAA_rekordy.xlsx"
    )

    assert (
        res.worksheets[0].cell(15, 3).value
        == "Czy z tytułu znikły wszystkie tagi HTML?"
    )



# Generated by Django 2.2.10 on 2020-03-16 19:27

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("raport_slotow", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="RaportUczelniaEwaluacjaView",
            fields=[
                (
                    "id",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.PositiveIntegerField(),
                        primary_key=True,
                        serialize=False,
                        size=4,
                    ),
                ),
                ("tytul_oryginalny", models.TextField()),
                (
                    "opis_bibliograficzny_autorzy_cache",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(), size=None
                    ),
                ),
                ("szczegoly", models.TextField()),
                ("informacje", models.TextField()),
                (
                    "autorzy_z_dyscypliny",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(), size=None
                    ),
                ),
                ("kolejnosc", models.PositiveSmallIntegerField()),
                ("punkty_kbn", models.DecimalField(decimal_places=2, max_digits=6)),
                (
                    "autor_dyscyplina_procent",
                    models.DecimalField(decimal_places=2, max_digits=5),
                ),
                (
                    "autor_subdyscyplina_procent",
                    models.DecimalField(decimal_places=2, max_digits=5),
                ),
                ("pkdaut", models.DecimalField(decimal_places=4, max_digits=20)),
                ("slot", models.DecimalField(decimal_places=4, max_digits=20)),
            ],
            options={
                "db_table": "bpp_uczelnia_ewaluacja_view",
                "ordering": ("tytul_oryginalny", "rekord_id", "kolejnosc"),
                "managed": False,
            },
        ),
        migrations.AlterModelOptions(
            name="raportzerowyentry",
            options={"managed": False, "ordering": ("autor", "dyscyplina_naukowa")},
        ),
    ]



from math import ceil

from django.db import models

from import_common.exceptions import XLSParseError
from import_common.util import XLSImportFile
from long_running.models import Operation
from long_running.notification_mixins import ASGINotificationMixin


class ImportOperation(ASGINotificationMixin, Operation):
    plik_xls = models.FileField()

    try_names = None
    banned_names = None
    min_points = None

    validation_form_class = None

    ignore_bad_rows = False

    def get_validation_form_class(self):
        return self.validation_form_class

    def get_xls_import_file(self):
        return XLSImportFile(
            self.plik_xls.path,
            try_names=self.try_names,
            banned_names=self.banned_names,
            min_points=self.min_points,
        )

    def perform(self):
        x = self.get_xls_import_file()

        total = x.count()
        five_percent = int(ceil(total / 50.0))
        form_class = self.get_validation_form_class()

        for no, elem in enumerate(x.data()):
            cleaned_data = None
            if form_class:
                form = form_class(elem)
                if not form.is_valid():
                    if self.ignore_bad_rows:
                        continue
                    raise XLSParseError(elem, form, "wstępna weryfikacja danych")

                cleaned_data = form.cleaned_data

            self.import_single_row(xls_data=elem, cleaned_data=cleaned_data)

            if no % five_percent == 0:
                self.send_progress(no * 100.0 / total)

    class Meta:
        abstract = True


class ImportRowMixin:
    def nr_arkusza(self):
        return self.dane_z_xls.get("__xls_loc_sheet__")

    def nr_wiersza(self):
        return self.dane_z_xls.get("__xls_loc_row__")



from django.contrib.postgres.search import SearchQuery


class SearchQueryStartsWith(SearchQuery):
    def as_sql(self, compiler, connection):
        value = self.source_expressions[1].value
        params = [value]
        if self.config:
            config_sql, config_params = compiler.compile(self.config)
            template = f"to_tsquery({config_sql}::regconfig, %s)"
            params = config_params + [value]
        else:
            template = "to_tsquery(%s)"
        if self.invert:
            template = f"!!({template})"
        return template, params



import contextlib

import pytest
from celery.result import AsyncResult
from django.urls import reverse

from rozbieznosci_dyscyplin.admin import (
    DYSCYPLINA_AUTORA,
    OFFLOAD_TASKS_WITH_THIS_ELEMENTS_OR_MORE,
    SUBDYSCYPLINA_AUTORA,
    RozbieznosciViewAdmin,
    parse_object_id,
    ustaw_druga_dyscypline,
    ustaw_dyscypline_task_or_instant,
    ustaw_pierwsza_dyscypline,
)
from rozbieznosci_dyscyplin.models import RozbieznosciView

from django.contrib.admin import AdminSite
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages import get_messages
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware

from bpp.models import Autor_Dyscyplina


@contextlib.contextmanager
def middleware(request):
    """Annotate a request object with a session"""
    middleware = SessionMiddleware()
    middleware.process_request(request)
    request.session.save()

    """Annotate a request object with a messages"""
    middleware = MessageMiddleware()
    middleware.process_request(request)
    request.session.save()
    yield request


@pytest.fixture
def zle_przypisana_praca(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
    wydawnictwo_ciagle,
    rok,
):
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        rok=rok,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    wca = wydawnictwo_ciagle.dodaj_autora(autor_jan_kowalski, jednostka)

    from django.db import connection

    cursor = connection.cursor()
    cursor.execute(
        "UPDATE bpp_wydawnictwo_ciagle_autor SET dyscyplina_naukowa_id = %s WHERE id = %s"
        % (dyscyplina3.pk, wca.pk)
    )

    # wca.dyscyplina_naukowa_id = dyscyplina3
    #     dyscyplina_naukowa=dyscyplina3)

    return wydawnictwo_ciagle


@pytest.mark.django_db
def test_znajdz_rozbieznosci_gdy_przypisanie_autor_dyscyplina(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
    wydawnictwo_ciagle,
    rok,
):
    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        rok=rok,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    wca = wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=dyscyplina1
    )

    assert RozbieznosciView.objects.count() == 0

    wca.dyscyplina_naukowa = dyscyplina2
    wca.save()

    assert RozbieznosciView.objects.count() == 0

    from django.db import connection

    cur = connection.cursor()
    cur.execute(
        "UPDATE bpp_wydawnictwo_ciagle_autor SET dyscyplina_naukowa_id = %s WHERE id = %s"
        % (dyscyplina3.pk, wca.pk)
    )

    assert RozbieznosciView.objects.first().autor == autor_jan_kowalski

    wca.dyscyplina_naukowa = None
    wca.save()

    assert RozbieznosciView.objects.first().autor == autor_jan_kowalski


@pytest.mark.django_db
def test_znajdz_rozbieznosci_bez_przypisania_autor_dyscyplina(
    autor_jan_kowalski,
    jednostka,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
    wydawnictwo_ciagle,
    rok,
):
    wca = wydawnictwo_ciagle.dodaj_autora(autor_jan_kowalski, jednostka)

    from django.db import connection

    cursor = connection.cursor()
    cursor.execute(
        "UPDATE bpp_wydawnictwo_ciagle_autor SET dyscyplina_naukowa_id = %s WHERE id = %s"
        % (dyscyplina1.pk, wca.pk)
    )

    assert RozbieznosciView.objects.count() == 1

    wca.dyscyplina_naukowa = None
    wca.save()

    assert RozbieznosciView.objects.count() == 0


@pytest.mark.django_db
def test_redirect_to_admin_view(wydawnictwo_ciagle, client, admin_user):
    res = client.get(
        reverse(
            "rozbieznosci_dyscyplin:redirect-to-admin",
            kwargs={
                "content_type_id": ContentType.objects.get_for_model(
                    wydawnictwo_ciagle
                ).pk,
                "object_id": wydawnictwo_ciagle.pk,
            },
        )
    )
    assert res.status_code == 302

    client.login(username=admin_user.username, password="password")
    res2 = client.get(res.url)

    assert res2.status_code == 200


@pytest.mark.django_db
def test_admin_usun_rozbieznosci_ustaw_pierwsza(zle_przypisana_praca, rf):
    assert RozbieznosciView.objects.count() == 1
    pk = str(RozbieznosciView.objects.first().pk)
    req = rf.post("/", data={"_selected_action": [pk]})

    with middleware(req):
        ustaw_pierwsza_dyscypline(None, req, None)
        msg = get_messages(req)

    assert RozbieznosciView.objects.count() == 0
    assert "ustawiono dyscyplinę" in list(msg)[0].message


@pytest.mark.django_db
def test_admin_usun_rozbieznosci_ustaw_druga(zle_przypisana_praca, rf):
    assert RozbieznosciView.objects.count() == 1
    pk = str(RozbieznosciView.objects.first().pk)
    req = rf.post("/", data={"_selected_action": [pk]})

    with middleware(req):
        ustaw_druga_dyscypline(None, req, None)
        assert RozbieznosciView.objects.count() == 0


@pytest.mark.django_db
def test_admin_usun_rozbieznosci_ustaw_pusta_druga(zle_przypisana_praca, rf):
    assert RozbieznosciView.objects.count() == 1

    ad = Autor_Dyscyplina.objects.get(
        autor=zle_przypisana_praca.autorzy.first(), rok=zle_przypisana_praca.rok
    )
    ad.subdyscyplina_naukowa = None
    ad.save()

    assert RozbieznosciView.objects.count() == 1
    pk = str(RozbieznosciView.objects.first().pk)
    req = rf.post("/", data={"_selected_action": [pk]})

    with middleware(req):
        ustaw_druga_dyscypline(None, req, None)
        msg = get_messages(req)

    assert "jest żadna" in list(msg)[0].message
    assert RozbieznosciView.objects.count() == 1


def test_RozbieznosciDyscyplinAdmin_przypisz_pierwsza_wszystkim(
    zle_przypisana_praca, rf, dyscyplina1
):
    ra = RozbieznosciViewAdmin(RozbieznosciView, AdminSite())
    req = rf.get("/")
    with middleware(req):
        ra.przypisz_wszystkim(req)
    assert RozbieznosciView.objects.count() == 0
    zle_przypisana_praca.refresh_from_db()
    assert zle_przypisana_praca.autorzy_set.first().dyscyplina_naukowa == dyscyplina1


def test_RozbieznosciDyscyplinAdmin_przypisz_druga_wszystkim(
    zle_przypisana_praca, rf, dyscyplina2
):
    ra = RozbieznosciViewAdmin(RozbieznosciView, AdminSite())
    req = rf.get("/")
    with middleware(req):
        ra.przypisz_druga_wszystkim(req)
    assert RozbieznosciView.objects.count() == 0
    zle_przypisana_praca.refresh_from_db()
    assert zle_przypisana_praca.autorzy_set.first().dyscyplina_naukowa == dyscyplina2


def test_RozbieznosciDyscyplinAdmin_test_task_offloading_offloads_dyscyplina(
    rf, zle_przypisana_praca, dyscyplina1
):
    req = rf.get("/")
    lst = [
        RozbieznosciView.objects.first().pk
    ] * OFFLOAD_TASKS_WITH_THIS_ELEMENTS_OR_MORE
    with middleware(req):
        ret = ustaw_dyscypline_task_or_instant(DYSCYPLINA_AUTORA, req, lst)
    assert isinstance(ret, AsyncResult)
    zle_przypisana_praca.refresh_from_db()
    assert zle_przypisana_praca.autorzy_set.first().dyscyplina_naukowa == dyscyplina1


def test_RozbieznosciDyscyplinAdmin_test_task_offloading_offloads_subdyscyplina(
    rf, zle_przypisana_praca, dyscyplina2
):
    req = rf.get("/")
    lst = [
        RozbieznosciView.objects.first().pk
    ] * OFFLOAD_TASKS_WITH_THIS_ELEMENTS_OR_MORE
    with middleware(req):
        ret = ustaw_dyscypline_task_or_instant(SUBDYSCYPLINA_AUTORA, req, lst)
    assert isinstance(ret, AsyncResult)
    zle_przypisana_praca.refresh_from_db()
    assert zle_przypisana_praca.autorzy_set.first().dyscyplina_naukowa == dyscyplina2


def test_RozbieznosciDyscyplinAdmin_test_task_offloading_instant_dyscyplina(
    rf, zle_przypisana_praca, dyscyplina1
):
    req = rf.get("/")
    lst = [RozbieznosciView.objects.first().pk] * (
        OFFLOAD_TASKS_WITH_THIS_ELEMENTS_OR_MORE - 1
    )
    with middleware(req):
        ret = ustaw_dyscypline_task_or_instant(DYSCYPLINA_AUTORA, req, lst)
    assert ret is None

    zle_przypisana_praca.refresh_from_db()
    assert zle_przypisana_praca.autorzy_set.first().dyscyplina_naukowa == dyscyplina1


def test_RozbieznosciDyscyplinAdmin_test_task_offloading_instant_subdyscyplina(
    rf, zle_przypisana_praca, dyscyplina2
):
    req = rf.get("/")
    lst = [RozbieznosciView.objects.first().pk] * (
        OFFLOAD_TASKS_WITH_THIS_ELEMENTS_OR_MORE - 1
    )
    with middleware(req):
        ret = ustaw_dyscypline_task_or_instant(SUBDYSCYPLINA_AUTORA, req, lst)
    assert ret is None

    zle_przypisana_praca.refresh_from_db()
    assert zle_przypisana_praca.autorzy_set.first().dyscyplina_naukowa == dyscyplina2


@pytest.mark.parametrize(
    "i,o",
    [
        ("(1,1,1)", [1, 1, 1]),
        ("asdf", None),
        ("(389489,34893489,4893398489)", [389489, 34893489, 4893398489]),
        ("(1,2,3,4)", None),
        ("[1,2,3]", [1, 2, 3]),
        ("{1:1,2:2,3:3}", None),
    ],
)
def test_parse_object_id(i, o):
    assert parse_object_id(i) == o


def test_RozbieznosciAutorZrodloAdmin(admin_app):
    res = admin_app.get(
        reverse("admin:rozbieznosci_dyscyplin_rozbieznoscizrodelview_changelist")
    )
    assert res.status_code == 200



# Generated by Django 2.1.7 on 2019-07-07 22:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0165_cache_punktacja_autora_cache_punktacja_dyscypliny_squashed_0167_auto_20190707_2029'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cache_punktacja_autora',
            options={'ordering': ('autor__nazwisko', 'dyscyplina__nazwa')},
        ),
        migrations.AlterModelOptions(
            name='cache_punktacja_dyscypliny',
            options={'ordering': ('dyscyplina__nazwa',)},
        ),
    ]



import pytest
from flexible_reports.models import Column, Datasource, Report, ReportElement, Table
from model_bakery import baker

from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType

from bpp.const import GR_RAPORTY_WYSWIETLANIE
from bpp.models import Wydawnictwo_Zwarte


@pytest.fixture
def generuj_raporty_app(app, normal_django_user):
    # To samo co ``app`` czyli aplikacja WebTest z zalogowanym ``normal_django_user``
    # ale użytkownik ma grupę "generuj raporty":
    normal_django_user.groups.add(
        Group.objects.get_or_create(name=GR_RAPORTY_WYSWIETLANIE)[0]
    )
    return app


@pytest.fixture
def raport_autorow(db):
    table = baker.make(Table)

    baker.make(Column, parent=table)

    ds = Datasource.objects.create(
        base_model=ContentType.objects.get_for_model(Wydawnictwo_Zwarte),
        dsl_query='tytul_oryginalny = "Wydawnictwo Zwarte"',
    )

    raport = Report.objects.create(
        slug="raport-autorow",
        title="Raport autorów",
        template="""
        "<h1>Raport autora - {{ object }} za
        {% if od_roku == do_roku %}
            rok {{ od_roku }}
        {% else %}
            lata {{ od_roku }} - {{ do_roku }}
        {% endif %}

        </h1>

        {% load django_tables2 %}

        <h2>1. Publikacje w czasopismach naukowych</h2>

        <h3>{{ elements.tabela_1_1.title }}</h3>
        {% render_table elements.tabela_1_1.table %}

        """,
    )

    ReportElement.objects.create(
        datasource=ds,
        table=table,
        slug="tabela_1_1",
        parent=raport,
        title="pani/jadzia",
    )

    return raport



# -*- encoding: utf-8 -*-


from django.core.exceptions import MultipleObjectsReturned
from django.db import models
from django.db import transaction
from django.db.models import Transform, CASCADE, SET_NULL

from bpp import fields
from bpp.models.zrodlo import Zrodlo, Punktacja_Zrodla
from integrator2.models.base import BaseIntegrationElement
from integrator2.util import read_xls_data
from .base import BaseIntegration


class UpperCase(Transform):
    lookup_name = 'upper'
    # bilateral = True # <- chyba w 1.8?
    # function = 'UPPER' <- j/w

    def as_sql(self, qn, connection):
        return "upper(" + self.lhs.as_sql(qn, connection)[0] + ")", []

from django.db.models import CharField, TextField

CharField.register_lookup(UpperCase)
TextField.register_lookup(UpperCase)


class ListaMinisterialnaElement(BaseIntegrationElement):
    parent = models.ForeignKey("ListaMinisterialnaIntegration", CASCADE)

    nazwa = models.TextField()
    issn = models.CharField("ISSN", max_length=32, blank=True, null=True)
    e_issn = models.CharField("e-ISSN", max_length=32, blank=True, null=True)
    punkty_kbn = models.IntegerField()
    zrodlo = models.ForeignKey(Zrodlo, SET_NULL, blank=True, null=True)

    class Meta:
        ordering = ['nazwa',]

    def __iter__(self):
        """Zwraca kolejne elementy do wyświetlenia na stronie w widoku tabeli.

        Kolejność zwracanych pól powinna odpowiadać kolejności pól nagłówka zwracanych przez
        ListaMinisterialnaIntegration.header_columns
        """
        return iter([self.nazwa, self.issn, self.e_issn, self.punkty_kbn])

def last_year():
    from datetime import datetime
    return datetime.now().date().year - 1


class ListaMinisterialnaIntegration(BaseIntegration):
    """Importuje wartości punktów KBN dla czasopism z list ministerialnych A, B, C.
    Wartości wejściowe w plikach to nazwa czasopisma, ISSN i opcjonalnie e-ISSN.
    """

    year = fields.YearField("Rok", default=last_year, help_text="Rok dla którego została wydana ta lista ministerialna")

    klass = ListaMinisterialnaElement

    class Meta:
        verbose_name = "integracja list ministerialnych"
        ordering = ['-uploaded_on']

    def input_file_to_dict_stream(self, limit=None, limit_sheets=None):
        gen = read_xls_data(
                self.file.path,
                {
                    "TYTUŁ CZASOPISMA": "nazwa",
                    "NR ISSN": "issn",
                    "NR E-ISSN": "e_issn",
                    "LICZBA PUNKTÓW ZA PUBLIKACJĘ W CZASOPIŚMIE NAUKOWYM ": "punkty_kbn"
                },
                "Lp.",
                limit=limit, limit_sheets=limit_sheets)
        next(gen)  # wiersz z cyferkami
        return gen

    def header_columns(self):
        return ["Nazwa", "ISSN", "E-ISSN", "Punkty"]

    def dict_stream_to_db(self, dict_stream=None, limit=None):
        if dict_stream is None:
            # Tylko z pierwszego sheet
            dict_stream = self.input_file_to_dict_stream(limit=limit, limit_sheets=1)

        for elem in dict_stream:
            ListaMinisterialnaElement.objects.create(
                    parent=self,
                    **dict([(str(x), y) for x, y in list(elem.items()) if x is not None and x != "__sheet__"]))
        pass

    strategie = [lambda obj: Zrodlo.objects.get(issn=obj.issn) if obj.issn is not None else None,
                 lambda obj: Zrodlo.objects.get(e_issn=obj.e_issn) if obj.e_issn is not None else None,
                 lambda obj: Zrodlo.objects.get(nazwa__upper=obj.nazwa.upper()) if obj.nazwa is not None else None]

    @transaction.atomic
    def match_single_record(self, elem):
        z = None
        for no, strategia in enumerate(self.strategie):
            try:
                z = strategia(elem)
                if z is not None:
                    break
            except MultipleObjectsReturned:
                elem.extra_info = "strategia %i zwraca > 1 obiekt" % no
                break
            except Zrodlo.DoesNotExist:
                continue
        if z:
            elem.zrodlo = z
            elem.moze_byc_zintegrowany_automatycznie = True
        else:
            elem.extra_info = "Brak takiego źródła w bazie danych"

    @transaction.atomic
    def integrate_single_record(self, elem):
        rok = elem.parent.year
        pz = Punktacja_Zrodla.objects.get_or_create(zrodlo=elem.zrodlo, rok=rok)[0]
        pz.punkty_kbn = elem.punkty_kbn
        pz.save()



# Generated by Django 2.1.7 on 2019-03-19 15:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0155_CASCADE'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent_autor',
            name='procent',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Udział w opracowaniu (procent)'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle_autor',
            name='procent',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Udział w opracowaniu (procent)'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte_autor',
            name='procent',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Udział w opracowaniu (procent)'),
        ),
    ]



from datetime import timedelta

from crossref.restful import Works
from django.db import models
from django.db.models import JSONField

from django.utils import timezone

from bpp.fields import DOIField


class CrossrefAPICacheManager(models.Manager):

    CACHE_DURATION_DAYS = 14
    cache_last_run = None

    def cleanup(self):
        cache_duration = timezone.now() - timedelta(days=self.CACHE_DURATION_DAYS)
        if self.cache_last_run is not None:
            if self.cache_last_run > cache_duration:
                return

        self.filter(ostatnio_zmodyfikowany__lte=cache_duration).delete()
        self.cache_last_run = timezone.now()

    def api_get_by_doi(self, doi):
        works = Works()
        return works.doi(doi)

    def get_by_doi(self, doi):
        self.cleanup()

        ret = self.filter(doi=doi).first()
        if ret is None:
            data = self.api_get_by_doi(doi)
            if data is None:
                return

            ret = self.create(doi=doi, data=data)

        return ret.data


class CrossrefAPICache(models.Model):
    objects = CrossrefAPICacheManager()

    doi = DOIField(unique=True)
    data = JSONField()
    ostatnio_zmodyfikowany = models.DateTimeField(auto_now=True)



from datetime import date, datetime

from django.core.exceptions import ValidationError
from django.test import TestCase
from model_bakery import baker

from bpp.models import (
    Autor,
    Jezyk,
    Patent,
    Plec,
    Punktacja_Zrodla,
    Redakcja_Zrodla,
    Typ_Odpowiedzialnosci,
    Tytul,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Ciagle_Autor,
    Zrodlo,
)
from bpp.models.autor import Autor_Jednostka, Funkcja_Autora
from bpp.models.system import Charakter_Formalny
from bpp.tests.util import (
    any_autor,
    any_ciagle,
    any_jednostka,
    any_uczelnia,
    any_wydzial,
    any_zwarte,
)


class TestNazwaISkrot(TestCase):
    def test_skrot(self):
        a = Charakter_Formalny(nazwa="foo", skrot="bar")
        self.assertEqual("foo", str(a))


class TestTytul(TestCase):
    def test_tytul(self):
        t = baker.make(Tytul)
        str(t)


class TestWydzial(TestCase):
    def test_wydzial(self):
        u = any_uczelnia()
        w = any_wydzial(nazwa="Lekarski", uczelnia=u)
        self.assertEqual(str(w), "Lekarski")


class TestJednostka(TestCase):
    def setUp(self):
        Typ_Odpowiedzialnosci.objects.get_or_create(nazwa="autor", skrot="aut.")
        Funkcja_Autora.objects.get_or_create(nazwa="kierownik", skrot="kier.")

    def test_jednostka(self):
        any_wydzial(skrot="BAR")
        j = any_jednostka(nazwa="foo", wydzial_skrot="BAR")
        self.assertEqual(str(j), "foo (BAR)")

    def test_obecni_autorzy(self):
        j1 = any_jednostka()

        a1 = baker.make(Autor)
        a2 = baker.make(Autor)
        a3 = baker.make(Autor)

        j1.dodaj_autora(a1, rozpoczal_prace=date(2012, 1, 1))
        j1.dodaj_autora(a2, zakonczyl_prace=date(2012, 12, 31))
        j1.dodaj_autora(a3)

        obecni = j1.obecni_autorzy()

        self.assertIn(a1, obecni)
        self.assertNotIn(a2, obecni)
        self.assertIn(a3, obecni)

    def test_kierownik(self):
        j1 = any_jednostka()
        self.assertEqual(j1.kierownik(), None)

        a1 = baker.make(Autor)
        j1.dodaj_autora(a1, funkcja=Funkcja_Autora.objects.get(nazwa="kierownik"))

        self.assertEqual(j1.kierownik(), a1)

    def test_prace_w_latach(self):
        j1 = any_jednostka()

        a1 = baker.make(Autor)

        wc = any_ciagle(rok=2012)
        Wydawnictwo_Ciagle_Autor.objects.create(
            rekord=wc,
            autor=a1,
            jednostka=j1,
            typ_odpowiedzialnosci=baker.make(Typ_Odpowiedzialnosci),
        )

        wc = any_ciagle(rok=2013)
        Wydawnictwo_Ciagle_Autor.objects.create(
            rekord=wc,
            autor=a1,
            jednostka=j1,
            typ_odpowiedzialnosci=baker.make(Typ_Odpowiedzialnosci),
        )

        self.assertEqual(list(j1.prace_w_latach()), [2012, 2013])


class TestAutor(TestCase):
    # fixtures = ['charakter_formalny.json', 'tytul.json',
    #             'typ_odpowiedzialnosci.json']

    def test_autor(self):
        j = baker.make(Autor, imiona="Omg", nazwisko="Lol", tytul=None)
        self.assertEqual(str(j), "Lol Omg")

        t = Tytul.objects.create(nazwa="daktur", skrot="dar")
        j = baker.make(Autor, imiona="Omg", nazwisko="Lol", tytul=t)
        self.assertEqual(str(j), "Lol Omg, dar")

        j.poprzednie_nazwiska = "Kowalski"
        self.assertEqual(str(j), "Lol Omg (Kowalski), dar")

        self.assertEqual(j.get_full_name(), "Omg Lol (Kowalski)")
        self.assertEqual(j.get_full_name_surname_first(), "Lol (Kowalski) Omg")

    def test_afiliacja_na_rok(self):
        w = any_wydzial()
        n = any_wydzial(skrot="w2", nazwa="w2")
        j = any_jednostka(wydzial=w)
        a = baker.make(Autor)

        aj = Autor_Jednostka.objects.create(
            autor=a, jednostka=j, funkcja=baker.make(Funkcja_Autora)
        )

        self.assertEqual(a.afiliacja_na_rok(2030, w), None)
        self.assertEqual(a.afiliacja_na_rok(2030, n), None)

        aj.rozpoczal_prace = date(2012, 1, 1)
        aj.save()

        self.assertEqual(a.afiliacja_na_rok(2030, w), True)
        self.assertEqual(a.afiliacja_na_rok(2011, w), None)
        self.assertEqual(a.afiliacja_na_rok(2030, n), None)

        aj.zakonczyl_prace = date(2013, 12, 31)
        aj.save()

        self.assertEqual(a.afiliacja_na_rok(2030, w), None)
        self.assertEqual(a.afiliacja_na_rok(2011, w), None)
        self.assertEqual(a.afiliacja_na_rok(2012, w), True)
        self.assertEqual(a.afiliacja_na_rok(2030, n), None)

    def test_dodaj_jednostke(self):
        f = baker.make(Funkcja_Autora)
        a = baker.make(Autor, imiona="Foo", nazwisko="Bar", tytul=None)
        w = any_wydzial(nazwa="WL", skrot="WL")
        w2 = any_wydzial(nazwa="XX", skrot="YY")
        j = any_jednostka(wydzial=w)
        j2 = any_jednostka(wydzial=w2)

        def ma_byc(ile=1):
            self.assertEqual(Autor_Jednostka.objects.count(), ile)

        a.dodaj_jednostke(j, 1912, f)
        ma_byc(1)

        a.dodaj_jednostke(j, 1913, f)
        ma_byc(1)

        a.dodaj_jednostke(j, 1914, f)
        ma_byc(1)

        a.dodaj_jednostke(j, 1913, f)
        ma_byc(1)

        a.dodaj_jednostke(j, 1920, f)
        ma_byc(2)

        a.dodaj_jednostke(j, 1921, f)
        ma_byc(2)

        a.dodaj_jednostke(j, 1960, f)
        ma_byc(3)

        lx = Autor_Jednostka.objects.all().order_by("rozpoczal_prace")
        self.assertEqual(lx[0].rozpoczal_prace, date(1912, 1, 1))
        self.assertEqual(lx[1].rozpoczal_prace, date(1920, 1, 1))

        self.assertEqual(lx[0].zakonczyl_prace, date(1914, 12, 31))
        self.assertEqual(lx[1].zakonczyl_prace, date(1921, 12, 31))

        self.assertEqual(a.afiliacja_na_rok(1912, w), True)
        self.assertEqual(a.afiliacja_na_rok(1913, w), True)
        self.assertEqual(a.afiliacja_na_rok(1914, w), True)
        self.assertEqual(a.afiliacja_na_rok(1914, w2), None)

        self.assertEqual(a.afiliacja_na_rok(1920, w), True)
        self.assertEqual(a.afiliacja_na_rok(1921, w), True)

        self.assertEqual(a.afiliacja_na_rok(1922, w), None)
        self.assertEqual(a.afiliacja_na_rok(1916, w), None)

        # Gdy jest wpisany tylko początek czasu pracy, traktujemy pracę
        # jako NIE zakończoną i każda data w przyszłości ma zwracać to miejsce
        Autor_Jednostka.objects.create(
            autor=a, jednostka=j2, rozpoczal_prace=date(2100, 1, 1)
        )
        self.assertEqual(a.afiliacja_na_rok(2200, w2), True)

        Autor_Jednostka.objects.all().delete()
        # Gdy nie ma dat początku i końca pracy, to funkcja ma zwracać NONE
        Autor_Jednostka.objects.create(autor=a, jednostka=j2)
        self.assertEqual(a.afiliacja_na_rok(2100, w2), None)
        self.assertEqual(a.afiliacja_na_rok(1100, w2), None)

    def test_autor_save(self):
        a = Autor.objects.create(nazwisko="von Foo", imiona="Bar")
        self.assertEqual(a.sort, "foobar")

    def test_autor_prace_w_latach(self):
        ROK = 2000

        a = baker.make(Autor)
        j = any_jednostka()
        w = baker.make(Wydawnictwo_Ciagle, rok=ROK)
        baker.make(Wydawnictwo_Ciagle_Autor, autor=a, jednostka=j, rekord=w)

        self.assertEqual(a.prace_w_latach()[0], ROK)


class TestAutor_Jednostka(TestCase):
    def setUp(self):
        Typ_Odpowiedzialnosci.objects.get_or_create(nazwa="autor", skrot="aut.")
        Funkcja_Autora.objects.get_or_create(skrot="kier.", nazwa="kierownik")

    def test_autor_jednostka(self):
        f = Funkcja_Autora.objects.get(skrot="kier.")
        a = baker.make(Autor, imiona="Omg", nazwisko="Lol", tytul=None)
        j = any_jednostka(nazwa="Lol", skrot="L.")
        aj = Autor_Jednostka.objects.create(autor=a, jednostka=j, funkcja=f)
        self.assertEqual(str(aj), "Lol Omg ↔ kierownik, L.")

        aj = Autor_Jednostka.objects.create(autor=a, jednostka=j, funkcja=None)
        self.assertEqual(str(aj), "Lol Omg ↔ L.")

        aj.rozpoczal_prace = datetime(2012, 1, 1)
        aj.zakonczyl_prace = datetime(2012, 1, 2)
        aj.full_clean()

        aj.rozpoczal_prace = datetime(2013, 1, 1)
        aj.zakonczyl_prace = datetime(2011, 1, 1)
        self.assertRaises(ValidationError, aj.full_clean)

    def test_defragmentuj(self):
        w = any_wydzial()
        a = baker.make(Autor)

        j1 = any_jednostka(nazwa="X", wydzial=w)
        any_jednostka(nazwa="Y", wydzial=w)
        any_jednostka(nazwa="Z", wydzial=w)

        # Taka sytuacja ma miejsce przy imporcie danych - w "starym" systemie są dane
        # na temat jednostek BEZ czasu pracy, zaś nowy dodaje informacje o czasie pracy
        Autor_Jednostka.objects.create(autor=a, jednostka=j1)
        Autor_Jednostka.objects.create(
            autor=a,
            jednostka=j1,
            rozpoczal_prace=date(2012, 1, 1),
            zakonczyl_prace=date(2012, 12, 31),
        )
        Autor_Jednostka.objects.create(
            autor=a,
            jednostka=j1,
            rozpoczal_prace=date(2013, 1, 1),
            zakonczyl_prace=date(2014, 12, 31),
        )
        Autor_Jednostka.objects.create(
            autor=a, jednostka=j1, rozpoczal_prace=date(2016, 1, 1)
        )

        Autor_Jednostka.objects.defragmentuj(a, j1)

        self.assertEqual(Autor_Jednostka.objects.all().count(), 2)

        Autor_Jednostka.objects.all().delete()

        # Ta sytuacja ma miejsce przy powtórnym imporcie XLSa do nowego systemu
        # autor ma pole rozpoczal_prace, ale zakoncyzl_prace jest NONE z powodu takiego, ze dalej
        # tam pracuje...
        Autor_Jednostka.objects.create(autor=a, jednostka=j1)
        Autor_Jednostka.objects.create(
            autor=a,
            jednostka=j1,
            rozpoczal_prace=date(2012, 1, 1),
            zakonczyl_prace=None,
        )
        Autor_Jednostka.objects.create(
            autor=a,
            jednostka=j1,
            rozpoczal_prace=date(2014, 1, 1),
            zakonczyl_prace=date(2015, 12, 31),
        )

        Autor_Jednostka.objects.defragmentuj(a, j1)
        aj = Autor_Jednostka.objects.all()[0]
        self.assertEqual(aj.rozpoczal_prace, date(2012, 1, 1))
        self.assertEqual(aj.zakonczyl_prace, date(2015, 12, 31))


class TestPunktacjaZrodla(TestCase):
    def test_punktacja_zrodla(self):
        z = baker.make(Zrodlo, nazwa="123 test")
        j = baker.make(Punktacja_Zrodla, rok="2012", impact_factor="0.5", zrodlo=z)
        self.assertEqual(str(j), 'Punktacja źródła "123 test" za rok 2012')


class TestZrodlo(TestCase):
    def test_zrodlo(self):
        z = baker.make(Zrodlo, nazwa="foo")
        self.assertEqual(str(z), "foo")

        z = baker.make(Zrodlo, nazwa="foo", nazwa_alternatywna="bar")
        self.assertEqual(str(z), "foo")

        z = baker.make(
            Zrodlo, nazwa="foo", poprzednia_nazwa="bar", nazwa_alternatywna="quux"
        )
        self.assertEqual(str(z), "foo (d. bar)")

        z = baker.make(Zrodlo, nazwa="foo", poprzednia_nazwa="quux")
        self.assertEqual(str(z), "foo (d. quux)")

    def test_zrodlo_prace_w_latach(self):
        z = baker.make(Zrodlo)
        any_ciagle(rok=2012, zrodlo=z)

        self.assertEqual(list(z.prace_w_latach()), [2012])

    def test_unicode(self):
        z = Zrodlo(nazwa="foo", poprzednia_nazwa="bar")
        self.assertEqual(str(z), "foo (d. bar)")


class TestRedakcjaZrodla(TestCase):
    # fixtures = ['plec.json']
    def setUp(self):
        Typ_Odpowiedzialnosci.objects.get_or_create(nazwa="autor", skrot="aut.")
        Plec.objects.get_or_create(skrot="M", nazwa="mężczyzna")
        Plec.objects.get_or_create(skrot="K", nazwa="kobieta")
        Tytul.objects.get_or_create(skrot="dr")

    def test_redakcja_zrodla(self):
        a = baker.make(
            Autor,
            imiona="Jan",
            nazwisko="Kowalski",
            tytul=Tytul.objects.get(skrot="dr"),
            plec=Plec.objects.get(skrot="M"),
        )
        z = baker.make(Zrodlo, nazwa="LOL Zine")
        r = Redakcja_Zrodla.objects.create(
            zrodlo=z, redaktor=a, od_roku=2010, do_roku=None
        )

        self.assertEqual(str(r), "Redaktorem od 2010 jest Kowalski Jan, dr")

        r.do_roku = 2012
        self.assertEqual(str(r), "Redaktorem od 2010 do 2012 był Kowalski Jan, dr")

        a.plec = Plec.objects.get(skrot="K")
        self.assertEqual(str(r), "Redaktorem od 2010 do 2012 była Kowalski Jan, dr")

        a.plec = None
        self.assertEqual(str(r), "Redaktorem od 2010 do 2012 był(a) Kowalski Jan, dr")


class TestAbstract(TestCase):
    def test_abstract(self):
        a = baker.make(Autor, imiona="Omg", nazwisko="Lol", tytul=None)

        j = any_jednostka(skrot="foo")
        t = baker.make(Typ_Odpowiedzialnosci, skrot="X", nazwa="Y")
        r = baker.make(Wydawnictwo_Ciagle, tytul_oryginalny="AAA")
        b = Wydawnictwo_Ciagle_Autor.objects.create(
            autor=a, jednostka=j, typ_odpowiedzialnosci=t, rekord=r
        )
        self.assertEqual(str(b), "Lol Omg - foo")
        self.assertEqual(str(r), "AAA")
        self.assertEqual(str(t), "Y")

    def test_ModelZNazwa(self):
        a = Jezyk.objects.create(nazwa="Foo", skrot="Bar")
        self.assertEqual(str(a), "Foo")


class TestTworzenieModeliAutorJednostka(TestCase):
    def setUp(self):
        Typ_Odpowiedzialnosci.objects.get_or_create(nazwa="autor", skrot="aut.")

    def test_tworzenie_modeli_autor_jednostka(self):
        a = any_autor()
        j = any_jednostka()
        c = any_ciagle()
        c.dodaj_autora(a, j)

        # Utworzenie modelu Wydawnictwo_Zwarte_Autor powinno utworzyć model
        # Autor_Jednostka, będący powiązaniem autora a z jednostką j
        self.assertEqual(Autor_Jednostka.objects.filter(autor=a).count(), 1)

    def test_tworzenie_modeli_autor_jednostka_zwarte(self):
        a = any_autor()
        j = any_jednostka()
        c = any_zwarte()
        c.dodaj_autora(a, j)

        # Utworzenie modelu Wydawnictwo_Zwarte_Autor powinno utworzyć model
        # Autor_Jednostka, będący powiązaniem autora a z jednostką j
        self.assertEqual(Autor_Jednostka.objects.filter(autor=a).count(), 1)


class TestWydawnictwoCiagleTestWydawnictwoZwarte(TestCase):
    # fixtures = ['charakter_formalny.json']

    def test_clean(self):
        for model in [any_ciagle, any_zwarte]:
            for skrot in ["PAT", "D", "H"]:
                try:
                    instance = model(
                        charakter_formalny=Charakter_Formalny.objects.get(skrot=skrot)
                    )
                except Charakter_Formalny.DoesNotExist:
                    continue

                self.assertRaises(ValidationError, instance.clean_fields)


def test_patent_kasowanie(autor_jan_kowalski, jednostka, typy_odpowiedzialnosci):
    p = baker.make(Patent)
    p.dodaj_autora(autor_jan_kowalski, jednostka)
    p.delete()



from bpp import const
from bpp.models.abstract import BazaModeluOdpowiedzialnosciAutorow


class WydawnictwoAutorToStatementPBNAdapter:
    def __init__(self, original: BazaModeluOdpowiedzialnosciAutorow):
        self.original = original

    def pbn_get_json(self):
        if (
            not self.original.afiliuje
            or not self.original.jednostka.skupia_pracownikow
            or self.original.jednostka.pk < 0
        ):
            return

        ret = {
            "type": const.TYP_OGOLNY_DO_PBN.get(
                self.original.typ_odpowiedzialnosci.typ_ogolny, "AUTHOR"
            ),
        }

        if self.original.profil_orcid:
            # To jest flaga dot. czy dane są w indeksie ORCID
            ret["orcid"] = True

        if self.original.dyscyplina_naukowa_id is not None and self.original.przypieta:
            ret["disciplineId"] = self.original.dyscyplina_naukowa.kod_dla_pbn()
        else:
            return

        # if self.original.jednostka.pbn_uid_id:
        #    ret["institutionId"] = self.original.jednostka.pbn_uid.pk

        if self.original.autor.pbn_uid_id:
            scientist = self.original.autor.pbn_uid

            pesel = scientist.value(
                "object", "externalIdentifiers", "PESEL", return_none=True
            )
            if pesel:
                ret["personNaturalId"] = pesel

            ret["personObjectId"] = scientist.pk

        else:
            if self.original.autor.orcid:  # and self.original.autor.orcid_w_pbn:
                ret["personOrcidId"] = self.original.autor.orcid

        if not ret.get("personOrcidId") and not ret.get("personObjectId"):
            return

        if self.original.data_oswiadczenia is not None:
            ret["statementDate"] = str(self.original.data_oswiadczenia)

        return ret



from django import forms

from zglos_publikacje.models import Zgloszenie_Publikacji


class ZwrocEmailForm(forms.ModelForm):
    class Meta:
        model = Zgloszenie_Publikacji
        fields = [
            "przyczyna_zwrotu",
        ]



import xml.etree.ElementTree as ET

import pytest
from django.urls.base import reverse


@pytest.mark.django_db
def test_proper_content_type(client):
    url = reverse("bpp:oai")
    url += "/oai-pmh-repository.xml?verb=Identify"
    res = client.get(url)
    assert "xml" in res["Content-type"]


@pytest.mark.django_db
def test_identify(wydawnictwo_ciagle, client):
    identify = reverse("bpp:oai") + "?verb=Identify"
    res = client.get(identify)
    assert res.status_code == 200


def toXML(response):
    return ET.fromstring(response.content.decode("utf-8"))


@pytest.mark.django_db
def test_listRecords(ksiazka, client):
    listRecords = reverse("bpp:oai") + "?verb=ListRecords&metadataPrefix=oai_dc"
    res = client.get(listRecords)

    ET.fromstring(res.content.decode("utf-8"))
    assert "Tytul Wydawnictwo" in toXML(res)[2][0][1][0][1].text


@pytest.mark.django_db
def test_listRecords_status_korekty(
    ksiazka, client, uczelnia, przed_korekta, po_korekcie
):
    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    ksiazka.status_korekty = przed_korekta
    ksiazka.save()

    listRecords = reverse("bpp:oai") + "?verb=ListRecords&metadataPrefix=oai_dc"
    res = toXML(client.get(listRecords))

    with pytest.raises(IndexError):
        assert "Tytul Wydawnictwo" in res[2][0][1][0][1].text

    ksiazka.status_korekty = po_korekcie
    ksiazka.save()
    listRecords = reverse("bpp:oai") + "?verb=ListRecords&metadataPrefix=oai_dc"
    res = toXML(client.get(listRecords))

    assert "Tytul Wydawnictwo" in res[2][0][1][0][1].text


@pytest.mark.django_db
def test_listRecords_no_queries_zwarte(ksiazka, client, django_assert_max_num_queries):
    listRecords = reverse("bpp:oai") + "?verb=ListRecords&metadataPrefix=oai_dc"
    with django_assert_max_num_queries(6):
        res = client.get(listRecords)
    assert "Tytul Wydawnictwo" in toXML(res)[2][0][1][0][1].text


@pytest.mark.django_db
def test_listRecords_no_queries_ciagle(artykul, client, django_assert_max_num_queries):
    listRecords = reverse("bpp:oai") + "?verb=ListRecords&metadataPrefix=oai_dc"
    with django_assert_max_num_queries(6):
        res = client.get(listRecords)
    assert "Tytul Wydawnictwo" in toXML(res)[2][0][1][0][1].text
